# -*- coding: utf-8 -*-
"""
Created on Thu Apr 26 12:58:45 2018

@author: kevin.stanton
"""

import math


class Shear_Calcs():

    def __init__(self, aData):
        '''
        This constructor defines all the input parameters required to assess
        the shear capacity of a reinforced concrete section
        '''

        # Parameters for faliure criteria
        self.fParams = aData.fParams
        self.ss_ratio_s = float(aData.ss_ratio_s)
        self.ss_ratio_t = float(aData.ss_ratio_t)
        self.c_condition = aData.c_condition
        self.rb_condition = aData.rb_condition

        # Global properties
        self.c = float(aData.cover)  # Cover of reinforcement bars (mm)
        self.codeOption = aData.codeOption
        self.concreteFactor = aData.concreteFactor
        self.gs_mat = aData.gs_mat  # Partial factor for steel

        # Define section properties
        self.shape = aData.section_type
        if self.shape == 'Circular':

            # Diameter of circular section (mm)
            self.D = float(aData.diameter_circ)

            # Diameter of reinforcement bars (mm)
            self.db = float(aData.bar_diam_circ)

            # Number of reinforcement bars
            self.n = float(aData.num_bars_circ)

            # Diameter oustide stirrups (mm)
            self.Ds = self.D - 2 * self.c

        elif self.shape == 'Cruciform':

            # Equivalent section geometry for rectangular section components
            self.bx = float(aData.bX)  # loading in x
            self.by = float(aData.tY)  # loading in x
            self.hx = float(aData.bY)  # loading in y
            self.hy = float(aData.tX)  # loading in y

            # Average diameter of reinforcement bars in tension side
            self.db_tx = (float(aData.cBar_hFlangeD) +
                          float(aData.cBar_centerD)) / 2
            self.db_ty = (float(aData.cBar_vFlangeD) +
                          float(aData.cBar_centerD)) / 2

            # Average diameter of reinforcement bars in compression side
            self.db_cx = (float(aData.cBar_hFlangeD) +
                          float(aData.cBar_centerD)) / 2
            self.db_cy = (float(aData.cBar_vFlangeD) +
                          float(aData.cBar_centerD)) / 2

            # Number of reinforcement bars in tension side
            nBar_center = int(aData.cBar_rows[0]) * int(aData.cBar_rows[1])
            self.n_tx = int(aData.cBar_hFlangeN) / 2 + nBar_center / 2
            self.n_ty = int(aData.cBar_vFlangeN) / 2 + nBar_center / 2

            # Number of reinforcement bars in compression side
            self.n_cx = int(aData.cBar_hFlangeN) / 2 + nBar_center / 2
            self.n_cy = int(aData.cBar_vFlangeN) / 2 + nBar_center / 2

        elif (self.shape == 'Rectangular Beam') |\
             (self.shape == 'Rectangular Column'):

            # Dimensions of rectangular section for shear in x (mm)
            self.bx = float(aData.rect_height)  # width
            self.by = float(aData.rect_width)  # height

            # Dimensions of rectangular section for shear in y (mm)
            self.hx = float(aData.rect_width)  # width
            self.hy = float(aData.rect_height)  # height

            # Diameter of reinforcement bars in tension side (mm)
            if self.shape == 'Rectangular Beam':
                self.db_tx = float(aData.db_b)
                self.db_ty = (float(aData.db_b) + float(aData.db_t)) / 2

            elif self.shape == 'Rectangular Column':
                self.db_tx = float(aData.bar_diam_col)
                self.db_ty = float(aData.bar_diam_col)

            # Number of reinforcement bars in tension side
            if self.shape == 'Rectangular Beam':
                self.n_tx = float(aData.n_b)
                self.n_ty = float(2)  # Assumes no side bars
            elif self.shape == 'Rectangular Column':
                if int(aData.num_bars_col) == 8:
                    self.n_tx = 3
                    self.n_ty = 3
                elif int(aData.num_bars_col) == 4:
                    self.n_tx = 2
                    self.n_ty = 2
                elif int(aData.num_bars_col) == 6:
                    self.n_tx = 2
                    self.n_ty = 3
                elif int(aData.num_bars_col) == 10:
                    self.n_tx = 4
                    self.n_ty = 4
                elif int(aData.num_bars_col) == 12:
                    self.n_tx = 4
                    self.n_ty = 4
                elif int(aData.num_bars_col) == 14:
                    self.n_tx = 5
                    self.n_ty = 5

            # Diameter of reinforcement bars in compression side (mm)
            if self.shape == 'Rectangular Beam':
                self.db_cx = float(aData.db_t)
                self.db_cy = (float(aData.db_b) + float(aData.db_t)) / 2
            elif self.shape == 'Rectangular Column':
                self.db_cx = float(aData.bar_diam_col)
                self.db_cy = float(aData.bar_diam_col)

            # Number of reinforcement bars in compression side
            if self.shape == 'Rectangular Beam':
                self.n_cx = float(aData.n_t)
                self.n_cy = float(2)  # Assuming there are no side bars
            elif self.shape == 'Rectangular Column':
                if int(aData.num_bars_col) == 8:
                    self.n_cx = 3
                    self.n_cy = 3
                elif int(aData.num_bars_col) == 4:
                    self.n_cx = 2
                    self.n_cy = 2
                elif int(aData.num_bars_col) == 6:
                    self.n_cx = 2
                    self.n_cy = 3
                elif int(aData.num_bars_col) == 10:
                    self.n_cx = 4
                    self.n_cy = 4
                elif int(aData.num_bars_col) == 12:
                    self.n_cx = 4
                    self.n_cy = 4
                elif int(aData.num_bars_col) == 14:
                    self.n_cx = 5
                    self.n_cy = 5

        elif self.shape == 'T Beam':

            # Dimensions of T Beam section for loading in x (mm)
            self.bx = float(aData.t_height)  # width
            self.by = float(aData.t_width)  # height

            # Dimensions of T Beam section for loading in y (mm)
            self.hx = float(aData.t_width)  # width
            self.hy = float(aData.t_height)  # height

            # Diameter of reinforcement bars in tension side (mm)
            self.db_tx = float(aData.t_bbar)
            self.db_ty = (float(aData.t_bbar) + float(aData.t_tbar)) / 2

            # Number of reinforcement bars in tension side
            self.n_tx = float(aData.t_nb)
            self.n_ty = float(2)  # Assuming there are no side bars

            # Diameter of reinforcement bars in compression side (mm)
            self.db_cx = float(aData.t_tbar)
            self.db_cy = (float(aData.t_bbar) + float(aData.t_tbar)) / 2

            # Number of reinforcement bars in compression side
            self.n_cx = float(aData.t_nt)
            self.n_cy = float(2)  # Assuming there are no side bars

        # Diameter of shear reinforcement (mm)
        self.ds = float(aData.d_shear)

        # Spacing of shear reinforcement (mm)
        self.s = float(aData.s_shear)

        # Number of legs pertaining to shear reinforcement in the section
        self.nsx = int(aData.n_legs[0])
        self.nsy = int(aData.n_legs[1])

        # Define material properties
        # Modulus of elasticity of steel reinforcement (GPa)
        self.Est = float(aData.Es) / 1000

        # Characteristic yield strength of reinforcement (MPa)
        self.fyk = float(aData.Fy)

        # Partial material safety factor for concrete
        self.gc = float(aData.gc_mat)

        # Partial material safety factor for steel
        self.gs = float(aData.gs_mat)

        # Cylindrical characteristic compressive strength of concrete
        self.fck = float(aData.Fc)

        # Cylindrical characteristic mean strength of concrete
        self.fcm = float(aData.Fc)

        # Design yield strength of shear reinforcement (N/mm2)
        self.fywd = float(aData.Fywd)

        # Maximum unusable compressive strain of concrete
        self.ecu = float(aData.ecu)

        # Coefficient for long term effects on compressive strength
        self.acc = 1.0

        # Yield strain of steel reinforcement
        self.ey = self.fyk / self.Est

        # Design value of concrete compressive strength (MPa)
        self.fcd = self.acc * self.fcm / self.gc

        # Define loading conditions
        # Applied moment in the section (kN-m)
        self.Med = float(aData.M_load)

        # Applied axial force in the section (kN)
        self.Ned = float(aData.N_load)

        # Applied shear force in the section (kN)
        self.Ved = float(aData.V_load)
        self.file_name = aData.file_name

        # Define reinforcement details
        if self.shape == 'Circular':

            # Reinforcement area in tension side (mm2)
            self.As_t = self.n / 2 * math.pi * self.db**2 / 4

            # Reinforcement area in compression side (mm2)
            self.As_c = self.As_t

            # Area of nonprestressed reinforcement (mm2)
            self.Asc = self.As_t + self.As_c
            self.Ascx = self.Asc
            self.Ascy = self.Asc

            # Net area of section (mm2)
            self.Anc = math.pi * self.D**2 / 4 - self.Asc
            self.Ancx = self.Anc
            self.Ancy = self.Anc

            # Gross section area (mm2)
            self.Ag = math.pi * self.D**2 / 4
            self.Agcx = self.Ag
            self.Agcy = self.Ag

            # Effective depth in tension side (mm)
            self.d = 0.5 * self.D + 0.5 * self.Ds * math.tan(2 / math.pi)

            # Gross section area (mm2)
            self.AgShearcx = self.D * self.s
            self.AgShearcy = self.D * self.s

        elif (self.shape == 'Rectangular Beam') |\
             (self.shape == 'Rectangular Column') |\
             (self.shape == 'T Beam') |\
             (self.shape == 'Cruciform'):

            # Reinforcement area in tension side (mm2)
            self.As_tx = self.n_tx * math.pi * self.db_tx**2 / 4
            self.As_ty = self.n_ty * math.pi * self.db_ty**2 / 4

            # Reinforcement area in compression side (mm2)
            self.As_cx = self.n_cx * math.pi * self.db_cx**2 / 4
            self.As_cy = self.n_cy * math.pi * self.db_cy**2 / 4

            # Area of nonprestressed reinforcement (mm2)
            self.Ascx = self.As_tx + self.As_cx
            self.Ascy = self.As_ty + self.As_cy

            # Net area of section (mm2)
            self.Ancx = self.bx * self.by - self.Ascx
            self.Ancy = self.hx * self.hy - self.Ascy

            # Gross section area (mm2)
            self.Agcx = self.bx * self.by
            self.Agcy = self.hx * self.hy

            # Effective depth in tension side (mm)
            self.d_tx = self.bx - self.c - self.db_tx / 2
            self.d_ty = self.by - self.c - self.db_ty / 2

            # Effective depth in compression side (mm)
            self.d_cx = self.c + self.db_cx / 2
            self.d_cy = self.c + self.db_cy / 2

            # Gross section area (mm2)
            self.AgShearcx = self.bx * self.s
            self.AgShearcy = self.hx * self.s

    def __Conc_Cap__(self, Ned, width, height, n_t, d_t, n_b, d_b, As_t, As_b, As_total, A_conc_net, d_eff_tens):
        '''
        This function calculates the shear resistance provided by the concrete
        '''
        
        if self.codeOption == 'EC2':

            # Non-dimensional coefficient (EC2 §6.2.2)
            CRd_c = 0.18 / self.gc
            if (self.shape == 'Rectangular Beam') |\
                (self.shape == 'Rectangular Column') |\
                (self.shape == 'T Beam') |\
                    (self.shape == 'Cruciform'):

                # Non-dimensional coefficient (EC2 §6.2.2)
                k = min(2, 1 + math.sqrt(200 / d_eff_tens))

                # Non-dimensional coefficient (EC2 §6.2.2)
                k1 = 0.15

                # Smallest width of cross-section in the tensile area (mm)
                bw = min(width, height)

                # Minimum value of concrete shear strength (EC2 §6.2.2) (MPa)
                vmin = 0.035 * k**(3 / 2) * self.fcm**0.5

                # Ratio of tensile reinforcement (Asl)
                ρl = min(0.02, As_t / (bw * d_eff_tens))

                # Normal stress acting on the concrete section (MPa)
                σcp = min(0.2 * self.fcd, Ned / 1000 / (A_conc_net / 1000000))

                # Design shear resistance of the concrete alone (kN)
                VRd_c = max((CRd_c * k * (100 * ρl * self.fcm)**(1 / 3) +
                              k1 * σcp) * (bw * d_eff_tens) / 1000,
                             (vmin + k1 * σcp) * (bw * d_eff_tens) / 1000)

            elif self.shape == 'Circular':

                α = (2 / math.pi) * (self.Ds / self.D)

                # Non-dimensional coefficient (EC2 §6.2.2)
                Av = 1 / 4 * self.D**2 * \
                    (math.pi / 2 + α + math.sin(α) * math.cos(α))
                k = min(2, 1 + math.sqrt(200 / self.d))

                # Ratio of tensile reinforcement (Asl)
                ρl = As_t / (0.25 * math.pi * 1000**2)

                # Design shear resistance of the concrete alone (kN)
                VRd_c = (CRd_c * k * (100 * ρl * self.fcm)
                          ** (1 / 3)) * Av / 1000

        elif self.codeOption == 'CSA':

            ### CSA A23.3-14 ###

            # Coefficients
            beta = 0.18  # CONSTANT
            phiC = 1 / self.gc
            if self.concreteFactor == 'normal':
                cFac = 1.0
            elif self.concreteFactor == 'semi-light':
                cFac = 0.85
            elif self.concreteFactor == 'light':
                cFac = 0.75

            if (self.shape == 'Rectangular Beam') |\
                (self.shape == 'Rectangular Column') |\
                (self.shape == 'T Beam') |\
                    (self.shape == 'Cruciform'):

                # Effective shear depth
                dv = max(0.9 * d_eff_tens, 0.72 * height)

                # Smallest width of cross-section in the tensile area (mm)
                bw = width

                ε = min(max(1 /
                              (2 *
                               self.Est *
                               As_t) *
                              (abs(0.0) /
                               (dv /
                                1000) +
                                  0.0 -
                                  Ned *
                                  0.5), 0), 0.003)
                beta = 0.18
                phiC = 1 / self.gc

                # Design shear resistance of the concrete alone (kN)
                VRd_c = (phiC * cFac * beta * (self.fcm)
                          ** (0.5)) * (bw * dv) / 1E3

            elif self.shape == 'Circular':
                # Effective shear depth
                dv = max(0.9 * self.d, 0.72 * self.D)

                # Axial strain at Ned
                ε = min(max(1 /
                              (2 *
                               self.Est *
                               As_t) *
                              (abs(0.0) /
                               (dv /
                                1000) +
                                  0.0 -
                                  Ned *
                                  0.5), 0), 0.003)

                beta = 0.18
                # Design shear resistance of the concrete alone (kN)
                VRd_c = (phiC * cFac * beta * (self.fcm)
                          ** (0.5)) * self.d * self.D / 1E3

        # Return output (kN)
        return VRd_c

    def __Rein_Cap__(self, Ned, width, height, n_s, d_s, s_spacing, n_t, d_t, n_b, d_b, As_t, As_b, As_total, A_conc_net, d_eff_tens):
        '''
        Calculates the shear resistance provided by the shear
        reinforcement
        '''
        
        # Cross-sectional area of the shear reinforcement (mm2)
        Asw = n_s * math.pi * (d_s**2) / 4
        self.Aswx = Asw # for legacy fParams logic
        self.Aswy = Asw # for legacy fParams logic

        if self.codeOption == 'EC2':

            # Strength reduction factor for concrete cracked in shear (EC2 §6.2.2)
            v = 0.6 * (1 - self.fcm / 250)

            if (self.shape == 'Rectangular Beam') |\
                (self.shape == 'Rectangular Column') |\
                (self.shape == 'T Beam') |\
                    (self.shape == 'Cruciform'):

                # Inner lever arm, for a member with constant depth (mm)
                z = 0.9 * d_eff_tens

                # Longitudinal strain at mid-depth (MC2010, 7.3-16)
                ε = min(max((1 / (2 * self.Est * As_t)) *
                              (abs(self.Med) / (z / 1000) + self.Ved - Ned * 0.5), 0), 0.003)

                # Αngle of concrete compression strut (MC2010, 7.3-39)
                # (degrees)
                θmin = 20 + 10000 * ε

                # Angle of Shear Reinforcement to Beam Axis (degrees)
                alpha = 90

                # Coefficient considering the state of stress in compression
                # chord
                acw = 1

                # Smallest width of cross-section in the tensile area (mm)
                bw = min(width, height)

                # Shear resistance due to crushing of compression struts (kN)
                VRd_max = abs(acw *
                               bw *
                               z *
                               v *
                               self.fcd /
                               (1000 *
                                (1 /
                                 math.tan(math. radians(θmin)) +
                                    1 /
                                    math.tan(math.radians(alpha)))))

                # Design shear resistance provided by shear reinforcement (kN)
                VRd_s = min(abs(Asw /
                                 s_pacing *
                                 z *
                                 self.fywd *
                                 (1 /
                                  math.tan(math.radians(θmin)) +
                                     1 /
                                     math.tan(math.radians(alpha))) /
                                 1000), VRd_max)

            elif self.shape == 'Circular':

                # Compute alpha
                α = (2 / math.pi) * (self.Ds / self.D)

                # Inner lever arm, for a member with constant depth (mm)
                z = max(0.9 * self.d, 0.72 * self.D)

                # Longitudinal strain at mid-depth (MC2010, 7.3-16)
                ε = min(max(1 / (2 * self.Est * As_t) * (self.Med / \
                          (z / 1000) + self.Ved - Ned * 0.5), 0), 0.003) # m/m

                # Αngle of concrete compression strut (MC2010, 7.3-39)
                # (degrees)
                θmin = 20 + 10000 * ε

                # Non-dimensional coefficient (EC2 §6.2.2)
                Av = 1 / 4 * self.D**2 * \
                    (math.pi / 2 + α + math.sin(α) * math.cos(α))

                # Shear resistance due to crushing of compression struts (kN)
                VRd_max = abs(v *
                               self.fcd *
                               Av *
                               z /
                               (1 /
                                math.tan(math.radians(θmin)) +
                                   math.tan(math.radians(θmin))) /
                               1000)                

                # Design shear resistance provided by shear reinforcement (kN)
                VRd_s = min(1.34 * Asw * self.fywd / s_spacing * self.D *
                             (1 / math.tan(math. radians(θmin))) / 1000, VRd_max)

        if self.codeOption == 'CSA':

            phiC = 1 / self.gc

            if (self.shape == 'Rectangular Beam') |\
                (self.shape == 'Rectangular Column') |\
                (self.shape == 'T Beam') |\
                    (self.shape == 'Cruciform'):

                # Inner lever arm, for a member with constant depth (mm)
                dv = max(0.9 * d_eff_tens, 0.72 * height)
                # Longitudinal strain at mid-depth
                ε = min(max(1 /
                              (2 *
                               self.Est *
                               As_t) *
                              (abs(0.0) /
                               (dv /
                                1000) +
                                  0.0 -
                                  Ned *
                                  0.5), 0), 0.003)

                # Αngle of concrete compression strut (degrees)
                θmin = 29 + 7000 * ε

                # Smallest width of cross-section in the tensile area (mm)
                bw = min(width, height)

                # Design shear resistance provided by shear reinforcement (kN)
                VRd_s = phiC * Asw * self.fywd * dv * \
                    (1 / math.tan(math.radians(θmin))) / (1000 * s_spacing)

            elif self.shape == 'Circular':

                # Compute alpha
                α = (2 / math.pi) * (self.Ds / self.D)

                # Inner lever arm, for a member with constant depth (mm)
                z = max(0.9 * self.d, 0.72 * self.D)

                # Longitudinal strain at mid-depth (MC2010, 7.3-16)
                ε = min(max(1 / (2 * self.Est * As_t) *
                              (0.0 / (z / 1000) + 0.0 - Ned * 0.5), 0), 0.003)

                # Αngle of concrete compression strut (MC2010, 7.3-39)
                # (degrees)
                θmin = 29 + 7000 * ε

                # Non-dimensional coefficient (EC2 §6.2.2)
                Av = 1 / 4 * self.D**2 * \
                    (math.pi / 2 + α + math.sin(α) * math.cos(α))

                # Design shear resistance provided by shear reinforcement (kN)
                VRd_s = phiC * Asw * self.fywd * z * \
                    (1 / math.tan(math.radians(θmin))) / (1000 * s_spacing)

        # Return output (kN)
        return VRd_s

    def Shear_cap(self, Ned, width, height, n_s, d_s, s_spacing, n_t, d_t, n_b, d_b):
        '''
        Calculates shear capacity of a reinforced concrete section
        '''
        
        # Area of steel in top/bottom (tension/compression)
        As_t = n_t * math.pi * (d_t/2)**2
        As_b = n_b * math.pi * (d_b/2)**2
        # Total steel area (mm2)
        As_total = As_t + As_b
        # Net concrete area of section (mm2)
        A_conc_net = width * height - As_total
        # Effective depth in tension side (mm)
        d_eff_tens = height - self.c - d_t / 2

        # Calculate shear capacity of concrete
        VRd_c = self.__Conc_Cap__(Ned, width, height, n_t, d_t, n_b, d_b, As_t, As_b, As_total, A_conc_net, d_eff_tens)

        # Calculate shear capacity of reinforcement
        VRd_s = self.__Rein_Cap__(Ned, width, height, n_s, d_s, s_spacing, n_t, d_t, n_b, d_b, As_t, As_b, As_total, A_conc_net, d_eff_tens)

        # Define shear capacity of reinforced concrete section
        if self.codeOption == 'EC2':
            VRd = max(VRd_c, VRd_s)
        if self.codeOption == 'CSA':
            phiC = 1 / self.gc
            if self.shape == 'Circular':
                dv = self.d
                bw = self.D
            else:
                dv = max(0.9 * d_eff_tens, 0.72 * height)
                bw = width
            VRd = min(VRd_c + VRd_s, 0.25 * phiC *
                       self.fcm * (bw * dv) / 1E3)

        # Return output
        return VRd

    def interpLin(self, y0, y1, x0, x1, x):
        y = y0 + (x - x0) * (y1 - y0) / (x1 - x0)
        return y

    def prLimits(self):
        '''
        Calculates plastic rotation thresholds following procedures from ASCE 41
        '''

        if self.fParams:
            if (self.Ved == 0) |\
                (self.Ved == 0) |\
                (math.isnan(self.Ved)):
                print(
                    'Warning: zero shear demand assumed (check inputs if unexpected)')
                self.Ved = 0.001
                
            if (self.shape == 'Rectangular Beam') |\
                    (self.shape == 'T Beam'):   
                # ASCE 41-17 Table 10-7
                # Note: ASCE 41-17 is the same as ASCE 41-13 for beams                
                if any("1" in i for i in self.rb_condition):

                    # Ratio of nonprestressed tension/compression reinforcement
                    # to net area for s/t-directions
                    rhoTs = self.As_tx / self.Ancx
                    rhoTt = self.As_ty / self.Ancy
                    rhoCs = self.As_cx / self.Ancx
                    rhoCt = self.As_cy / self.Ancy

                    # Compute reinforcement ratio producing balanced strain
                    if self.fck <= 30:
                        beta = 0.85
                    else:
                        beta = max(0.85 - 0.05 / (7 * (self.fck - 30)), 0.65)
                    self.rho_bal_s = 0.85 * self.fck * beta * self.ecu * self.Est * \
                        0.001 / (self.fyk * (self.ecu * self.Est * 0.001 + self.fyk))
                    self.rho_bal_t = self.rho_bal_s

                    # Determine balance point in s/t-directions
                    bP_s = (rhoTs - rhoCs) / self.rho_bal_s
                    bP_t = (rhoTt - rhoCt) / self.rho_bal_t

                    # Determine compliance of transverse reinforcement
                    self.b = max(self.d_tx, self.d_ty)  # Governing dimension
                    if self.s <= self.b / 3:
                        compliance_t = 'C'
                        compliance_s = 'C'
                    else:
                        compliance_s = 'NC'
                        compliance_t = 'NC'

                    # Determine shear constant
                    vC_s = self.Ved * 1000 / \
                        (self.by * self.bx * 0.8 * (self.fck)**0.5)
                    vC_t = self.Ved * 1000 / \
                        (self.bx * self.by * 0.8 * (self.fck)**0.5)

                    # Determine rotation limits defining failure in the local
                    # s-direction based on Table 10-7 (ASCE 41-17)
                    if compliance_s == 'C':
                        if vC_s <= 0.25 and bP_s <= 0:
                            # If row 1
                            c1_PRS1, c1_PRS2, c1_PRS3, c1_PRS4 = 0, 0.01, 0.025, 0.05
                        elif vC_s >= 0.25 and vC_s <= 0.5 and bP_s <= 0:
                            # If between row 1 and 2 for vC_s
                            c1_PRS1 = 0
                            c1_PRS2 = self.interpLin(
                                0.01, 0.005, 0.25, 0.5, vC_s)
                            c1_PRS3 = self.interpLin(
                                0.025, 0.02, 0.25, 0.5, vC_s)
                            c1_PRS4 = self.interpLin(
                                0.05, 0.04, 0.25, 0.5, vC_s)
                        elif vC_s >= 0.5 and bP_s <= 0:
                            # If row 2
                            c1_PRS1, c1_PRS2, c1_PRS3, c1_PRS4 = 0, 0.005, 0.02, 0.04
                        elif vC_s <= 0.25 and bP_s >= 0 and bP_s <= 0.5:
                            # If between row 1 and 3 for bP_s
                            c1_PRS1 = 0
                            c1_PRS2 = self.interpLin(0.01, 0.005, 0, 0.5, bP_s)
                            c1_PRS3 = self.interpLin(0.025, 0.02, 0, 0.5, bP_s)
                            c1_PRS4 = self.interpLin(0.05, 0.03, 0, 0.5, bP_s)
                        elif vC_s <= 0.25 and bP_s >= 0.5:
                            # If row 3
                            c1_PRS1, c1_PRS2, c1_PRS3, c1_PRS4 = 0, 0.005, 0.02, 0.03
                        elif vC_s >= 0.5 and bP_s >= 0 and bP_s <= 0.5:
                            # If between row 2 and 4 for bP_s
                            c1_PRS1 = 0
                            c1_PRS2 = 0.005
                            c1_PRS3 = self.interpLin(0.02, 0.015, 0, 0.5, bP_s)
                            c1_PRS4 = self.interpLin(0.04, 0.04, 0, 0.5, bP_s)
                        elif vC_s >= 0.25 and vC_s <= 0.5 and bP_s >= 0.5:
                            # If between row 3 and 4 for vC_s
                            c1_PRS1 = 0
                            c1_PRS2 = 0.005
                            c1_PRS3 = self.interpLin(
                                0.02, 0.015, 0.25, 0.5, bP_s)
                            c1_PRS4 = self.interpLin(
                                0.03, 0.02, 0.25, 0.5, bP_s)
                        elif vC_s >= 0.5 and bP_s >= 0.5:
                            # If row 4
                            c1_PRS1, c1_PRS2, c1_PRS3, c1_PRS4 = 0, 0.005, 0.015, 0.02

                    elif compliance_s == 'NC':
                        if vC_s <= 0.25 and bP_s <= 0:
                            # If row 1
                            c1_PRS1, c1_PRS2, c1_PRS3, c1_PRS4 = 0, 0.005, 0.02, 0.03
                        elif vC_s >= 0.25 and vC_s <= 0.5 and bP_s <= 0:
                            # If between row 1 and 2 for vC_s
                            c1_PRS1 = 0
                            c1_PRS2 = self.interpLin(
                                0.005, 0.0015, 0, 0.5, bP_s)
                            c1_PRS3 = self.interpLin(0.02, 0.01, 0, 0.5, bP_s)
                            c1_PRS4 = self.interpLin(0.03, 0.015, 0, 0.5, bP_s)
                        elif vC_s >= 0.5 and bP_s <= 0:
                            # If row 2
                            c1_PRS1, c1_PRS2, c1_PRS3, c1_PRS4 = 0, 0.0015, 0.01, 0.015
                        elif vC_s <= 0.25 and bP_s >= 0 and bP_s <= 0.5:
                            # If between row 1 and 3 for bP_s
                            c1_PRS1 = 0
                            c1_PRS2 = 0.005
                            c1_PRS3 = self.interpLin(0.02, 0.01, 0, 0.5, bP_s)
                            c1_PRS4 = self.interpLin(0.03, 0.015, 0, 0.5, bP_s)
                        elif vC_s <= 0.25 and bP_s >= 0.5:
                            # If row 3
                            c1_PRS1, c1_PRS2, c1_PRS3, c1_PRS4 = 0, 0.005, 0.01, 0.015
                        elif vC_s >= 0.5 and bP_s >= 0 and bP_s <= 0.5:
                            # If between row 2 and 4 for bP_s
                            c1_PRS1 = 0
                            c1_PRS2 = 0.0015
                            c1_PRS3 = self.interpLin(0.01, 0.005, 0, 0.5, bP_s)
                            c1_PRS4 = self.interpLin(0.015, 0.01, 0, 0.5, bP_s)
                        elif vC_s >= 0.25 and vC_s <= 0.5 and bP_s >= 0.5:
                            # If between row 3 and 4 for vC_s
                            c1_PRS1 = 0
                            c1_PRS2 = self.interpLin(
                                0.005, 0.0015, 0.25, 0.5, bP_s)
                            c1_PRS3 = self.interpLin(
                                0.01, 0.005, 0.25, 0.5, bP_s)
                            c1_PRS4 = self.interpLin(
                                0.015, 0.01, 0.25, 0.5, bP_s)
                        elif vC_s >= 0.5 and bP_s >= 0.5:
                            # If row 4
                            c1_PRS1, c1_PRS2, c1_PRS3, c1_PRS4 = 0, 0.0015, 0.005, 0.01

                    if compliance_t == 'C':
                        if vC_t <= 0.25 and bP_t <= 0:
                            # If row 1
                            c1_PRT1, c1_PRT2, c1_PRT3, c1_PRT4 = 0, 0.01, 0.025, 0.05
                        elif vC_t >= 0.25 and vC_t <= 0.5 and bP_t <= 0:
                            # If between row 1 and 2 for vC_t
                            c1_PRT1 = 0
                            c1_PRT2 = self.interpLin(
                                0.01, 0.005, 0.25, 0.5, vC_t)
                            c1_PRT3 = self.interpLin(
                                0.025, 0.02, 0.25, 0.5, vC_t)
                            c1_PRT4 = self.interpLin(
                                0.05, 0.04, 0.25, 0.5, vC_t)
                        elif vC_t >= 0.5 and bP_t <= 0:
                            # If row 2
                            c1_PRT1, c1_PRT2, c1_PRT3, c1_PRT4 = 0, 0.005, 0.02, 0.04
                        elif vC_t <= 0.25 and bP_t >= 0 and bP_t <= 0.5:
                            # If between row 1 and 3 for bP_t
                            c1_PRT1 = 0
                            c1_PRT2 = self.interpLin(0.01, 0.005, 0, 0.5, bP_t)
                            c1_PRT3 = self.interpLin(0.025, 0.02, 0, 0.5, bP_t)
                            c1_PRT4 = self.interpLin(0.05, 0.03, 0, 0.5, bP_t)
                        elif vC_t <= 0.25 and bP_t >= 0.5:
                            # If row 3
                            c1_PRT1, c1_PRT2, c1_PRT3, c1_PRT4 = 0, 0.005, 0.02, 0.03
                        elif vC_t >= 0.5 and bP_t >= 0 and bP_t <= 0.5:
                            # If between row 2 and 4 for bP_t
                            c1_PRT1 = 0
                            c1_PRT2 = 0.005
                            c1_PRT3 = self.interpLin(0.02, 0.015, 0, 0.5, bP_t)
                            c1_PRT4 = self.interpLin(0.04, 0.04, 0, 0.5, bP_t)
                        elif vC_t >= 0.25 and vC_t <= 0.5 and bP_t >= 0.5:
                            # If between row 3 and 4 for vC_t
                            c1_PRT1 = 0
                            c1_PRT2 = 0.005
                            c1_PRT3 = self.interpLin(
                                0.02, 0.015, 0.25, 0.5, bP_t)
                            c1_PRT4 = self.interpLin(
                                0.03, 0.02, 0.25, 0.5, bP_t)
                        elif vC_t >= 0.5 and bP_t >= 0.5:
                            # If row 4
                            c1_PRT1, c1_PRT2, c1_PRT3, c1_PRT4 = 0, 0.005, 0.015, 0.02

                    elif compliance_t == 'NC':
                        if vC_t <= 0.25 and bP_t <= 0:
                            # If row 1
                            c1_PRT1, c1_PRT2, c1_PRT3, c1_PRT4 = 0, 0.005, 0.02, 0.03
                        elif vC_t >= 0.25 and vC_t <= 0.5 and bP_t <= 0:
                            # If between row 1 and 2 for vC_t
                            c1_PRT1 = 0
                            c1_PRT2 = self.interpLin(
                                0.005, 0.0015, 0, 0.5, bP_t)
                            c1_PRT3 = self.interpLin(0.02, 0.01, 0, 0.5, bP_t)
                            c1_PRT4 = self.interpLin(0.03, 0.015, 0, 0.5, bP_t)
                        elif vC_t >= 0.5 and bP_t <= 0:
                            # If row 2
                            c1_PRT1, c1_PRT2, c1_PRT3, c1_PRT4 = 0, 0.0015, 0.01, 0.015
                        elif vC_t <= 0.25 and bP_t >= 0 and bP_t <= 0.5:
                            # If between row 1 and 3 for bP_t
                            c1_PRT1 = 0
                            c1_PRT2 = 0.005
                            c1_PRT3 = self.interpLin(0.02, 0.01, 0, 0.5, bP_t)
                            c1_PRT4 = self.interpLin(0.03, 0.015, 0, 0.5, bP_t)
                        elif vC_t <= 0.25 and bP_t >= 0.5:
                            # If row 3
                            c1_PRT1, c1_PRT2, c1_PRT3, c1_PRT4 = 0, 0.005, 0.01, 0.015
                        elif vC_t >= 0.5 and bP_t >= 0 and bP_t <= 0.5:
                            # If between row 2 and 4 for bP_t
                            c1_PRT1 = 0
                            c1_PRT2 = 0.0015
                            c1_PRT3 = self.interpLin(0.01, 0.005, 0, 0.5, bP_t)
                            c1_PRT4 = self.interpLin(0.015, 0.01, 0, 0.5, bP_t)
                        elif vC_t >= 0.25 and vC_t <= 0.5 and bP_t >= 0.5:
                            # If between row 3 and 4 for vC_t
                            c1_PRT1 = 0
                            c1_PRT2 = self.interpLin(
                                0.005, 0.0015, 0.25, 0.5, bP_t)
                            c1_PRT3 = self.interpLin(
                                0.01, 0.005, 0.25, 0.5, bP_t)
                            c1_PRT4 = self.interpLin(
                                0.015, 0.01, 0.25, 0.5, bP_t)
                        elif vC_t >= 0.5 and bP_t >= 0.5:
                            # If row 4
                            c1_PRT1, c1_PRT2, c1_PRT3, c1_PRT4 = 0, 0.0015, 0.005, 0.01

                if any("2" in i for i in self.rb_condition):
                    if self.s <= self.bx / 2:
                        c2_PRS1, c2_PRS2, c2_PRS3, c2_PRS4 = 0, 0.0015, 0.01, 0.02
                    else:
                        c2_PRS1, c2_PRS2, c2_PRS3, c2_PRS4 = 0, 0.0015, 0.005, 0.01
                    if self.s <= self.by / 2:
                        c2_PRT1, c2_PRT2, c2_PRT3, c2_PRT4 = 0, 0.0015, 0.01, 0.02
                    else:
                        c2_PRT1, c2_PRT2, c2_PRT3, c2_PRT4 = 0, 0.0015, 0.005, 0.01

                if any("3" in i for i in self.rb_condition):
                    if self.s <= self.bx / 2:
                        c3_PRS1, c3_PRS2, c3_PRS3, c3_PRS4 = 0, 0.0015, 0.01, 0.02
                    else:
                        c3_PRS1, c3_PRS2, c3_PRS3, c3_PRS4 = 0, 0.0015, 0.005, 0.01
                    if self.s <= self.by / 2:
                        c3_PRT1, c3_PRT2, c3_PRT3, c3_PRT4 = 0, 0.0015, 0.01, 0.02
                    else:
                        c3_PRT1, c3_PRT2, c3_PRT3, c3_PRT4 = 0, 0.0015, 0.005, 0.01

                if any("4" in i for i in self.rb_condition):
                    c4_PRS1, c4_PRS2, c4_PRS3, c4_PRS4 = 0, 0.01, 0.02, 0.03
                    c4_PRT1, c4_PRT2, c4_PRT3, c4_PRT4 = 0, 0.01, 0.02, 0.03

                # Specify dummy values for conditions that do not control
                if not any("1" in i for i in self.rb_condition):
                    c1_PRS1, c1_PRS2, c1_PRS3, c1_PRS4 = 1, 1, 1, 1
                    c1_PRT1, c1_PRT2, c1_PRT3, c1_PRT4 = 1, 1, 1, 1
                if not any("2" in i for i in self.rb_condition):
                    c2_PRS1, c2_PRS2, c2_PRS3, c2_PRS4 = 1, 1, 1, 1
                    c2_PRT1, c2_PRT2, c2_PRT3, c2_PRT4 = 1, 1, 1, 1
                if not any("3" in i for i in self.rb_condition):
                    c3_PRS1, c3_PRS2, c3_PRS3, c3_PRS4 = 1, 1, 1, 1
                    c3_PRT1, c3_PRT2, c3_PRT3, c3_PRT4 = 1, 1, 1, 1
                if not any("4" in i for i in self.rb_condition):
                    c4_PRS1, c4_PRS2, c4_PRS3, c4_PRS4 = 1, 1, 1, 1
                    c4_PRT1, c4_PRT2, c4_PRT3, c4_PRT4 = 1, 1, 1, 1

                # Take the minimum outcomes among all specified conditions
                PRS1, PRS2, PRS3, PRS4 = [
                    min(
                        c1_PRS1, c2_PRS1, c3_PRS1, c4_PRS1), min(
                        c1_PRS2, c2_PRS2, c3_PRS2, c4_PRS2), min(
                        c1_PRS3, c2_PRS3, c3_PRS3, c4_PRS3), min(
                        c1_PRS4, c2_PRS4, c3_PRS4, c4_PRS4)]
                PRT1, PRT2, PRT3, PRT4 = [
                    min(
                        c1_PRT1, c2_PRT1, c3_PRT1, c4_PRT1), min(
                        c1_PRT2, c2_PRT2, c3_PRT2, c4_PRT2), min(
                        c1_PRT3, c2_PRT3, c3_PRT3, c4_PRT3), min(
                        c1_PRT4, c2_PRT4, c3_PRT4, c4_PRT4)]

            if (self.shape == 'Circular') |\
                    (self.shape == 'Rectangular Column') |\
                    (self.shape == 'Cruciform'):
                
                if self.fParams == 'ASCE 41-13':
                    # General parameters for ASCE 41-13 Table 10-8
                    paf = self.Ned/(self.Ag*self.fck)
                    rho_s = self.AgShearcx/(self.bx*self.s)
                    rho_t = self.AgShearcy/(self.by*self.s)
                    # Determine shear constant
                    vC_s = self.Ved * 1000 / \
                        (self.by * 0.8 * self.bx * (self.fck)**0.5)
                    vC_t = self.Ved * 1000 / \
                        (self.bx * 0.8 * self.by * (self.fck)**0.5)
                    
                    if any("1" in i for i in self.c_condition):
                        c1_PRS1 = 0
                        c1_PRT1 = 0
                        if paf <= 0.1:
                            if rho_s >= 0.006:
                                c1_PRS1, c1_PRS2, c1_PRS3, c1_PRS4 = 0, 0.005, 0.045, 0.06
                            elif rho_s <= 0.002:
                                c1_PRS1, c1_PRS2, c1_PRS3, c1_PRS4 = 0, 0.005, 0.027, 0.034
                            else:
                                c1_PRS2 = 0.005
                                c1_PRS3 = self.interpLin(0.027, 0.045, 0.002, 0.006, rho_s)
                                c1_PRS4 = self.interpLin(0.034, 0.06, 0.002, 0.006, rho_s)
                            if rho_t >= 0.006:
                                c1_PRT1, c1_PRT2, c1_PRT3, c1_PRT4 = 0, 0.005, 0.045, 0.06                                
                            elif rho_t <= 0.002:
                                c1_PRT1, c1_PRT2, c1_PRT3, c1_PRT4 = 0, 0.005, 0.027, 0.034
                            else:
                                c1_PRT2 = 0.005
                                c1_PRT3 = self.interpLin(0.027, 0.045, 0.002, 0.006, rho_t)
                                c1_PRT4 = self.interpLin(0.034, 0.06, 0.002, 0.006, rho_t)
                        if paf >= 0.6:
                            if rho_s >= 0.006:
                                c1_PRS1, c1_PRS2, c1_PRS3, c1_PRS4 = 0, 0.003, 0.009, 0.01
                            elif rho_s <= 0.002:
                                c1_PRS1, c1_PRS2, c1_PRS3, c1_PRS4 = 0, 0.002, 0.004, 0.005
                            else:
                                c1_PRS2 = self.interpLin(0.002, 0.003, 0.002, 0.006, rho_s)
                                c1_PRS3 = self.interpLin(0.004, 0.009, 0.002, 0.006, rho_s)
                                c1_PRS4 = self.interpLin(0.005, 0.01, 0.002, 0.006, rho_s)
                            if rho_t >= 0.006:
                                c1_PRT1, c1_PRT2, c1_PRT3, c1_PRT4 = 0, 0.003, 0.009, 0.01                               
                            elif rho_t <= 0.002:
                                c1_PRT1, c1_PRT2, c1_PRT3, c1_PRT4 = 0, 0.002, 0.004, 0.005
                            else:
                                c1_PRT2 = self.interpLin(0.002, 0.003, 0.002, 0.006, rho_t)
                                c1_PRT3 = self.interpLin(0.004, 0.009, 0.002, 0.006, rho_t)
                                c1_PRT4 = self.interpLin(0.005, 0.01, 0.002, 0.006, rho_t)
                        else:
                            c1_PRS2_g = self.interpLin(0.003, 0.005, 0.6, 0.1, paf)
                            c1_PRS3_g = self.interpLin(0.009, 0.045, 0.6, 0.1, paf)
                            c1_PRS4_g = self.interpLin(0.01, 0.06, 0.6, 0.1, paf)
                            c1_PRS2_l = self.interpLin(0.002, 0.005, 0.6, 0.1, paf)
                            c1_PRS3_l = self.interpLin(0.004, 0.027, 0.6, 0.1, paf)
                            c1_PRS4_l = self.interpLin(0.005, 0.034, 0.6, 0.1, paf)
                            c1_PRT2_g = self.interpLin(0.003, 0.005, 0.6, 0.1, paf)
                            c1_PRT3_g = self.interpLin(0.009, 0.045, 0.6, 0.1, paf)
                            c1_PRT4_g = self.interpLin(0.01, 0.06, 0.6, 0.1, paf)
                            c1_PRT2_l = self.interpLin(0.002, 0.005, 0.6, 0.1, paf)
                            c1_PRT3_l = self.interpLin(0.004, 0.027, 0.6, 0.1, paf)
                            c1_PRT4_l = self.interpLin(0.005, 0.034, 0.6, 0.1, paf)
                            if rho_s >= 0.006:
                                c1_PRS2 = c1_PRS2_g
                                c1_PRS3 = c1_PRS3_g
                                c1_PRS4 = c1_PRS4_g
                            elif rho_s <= 0.002:
                                c1_PRS2 = c1_PRS2_l
                                c1_PRS3 = c1_PRS3_l
                                c1_PRS4 = c1_PRS4_l
                            else:
                                c1_PRS2 = self.interpLin(c1_PRS2_l, c1_PRS2_g, 0.002, 0.006, rho_s)
                                c1_PRS3 = self.interpLin(c1_PRS3_l, c1_PRS3_g, 0.002, 0.006, rho_s)
                                c1_PRS4 = self.interpLin(c1_PRS4_l, c1_PRS4_g, 0.002, 0.006, rho_s)
                            if rho_t >= 0.006:
                                c1_PRT2 = c1_PRT2_g
                                c1_PRT3 = c1_PRT3_g
                                c1_PRT4 = c1_PRT4_g
                            elif rho_t <= 0.002:
                                c1_PRT2 = c1_PRT2_l
                                c1_PRT3 = c1_PRT3_l
                                c1_PRT4 = c1_PRT4_l
                            else:
                                c1_PRT2 = self.interpLin(c1_PRT2_l, c1_PRT2_g, 0.002, 0.006, rho_t)
                                c1_PRT3 = self.interpLin(c1_PRT3_l, c1_PRT3_g, 0.002, 0.006, rho_t)
                                c1_PRT4 = self.interpLin(c1_PRT4_l, c1_PRT4_g, 0.002, 0.006, rho_t)
                                
                    if any("2" in i for i in self.c_condition):
                        c2_PRS1 = 0
                        c2_PRT1 = 0
                        if paf <= 0.1:
                            if rho_s >= 0.006:
                                c2_PRS1_vl, c2_PRS2_vl, c2_PRS3_vl, c2_PRS4_vl = 0, 0.005, 0.045, 0.06
                                c2_PRS1_vg, c2_PRS2_vg, c2_PRS3_vg, c2_PRS4_vg = 0, 0.005, 0.045, 0.06
                            elif rho_s <= 0.0005:
                                c2_PRS1_vl, c2_PRS2_vl, c2_PRS3_vl, c2_PRS4_vl = 0, 0.005, 0.01, 0.012
                                c2_PRS1_vg, c2_PRS2_vg, c2_PRS3_vg, c2_PRS4_vg = 0, 0.004, 0.005, 0.006
                            else:
                                c2_PRS2_vl = 0.005
                                c2_PRS3_vl = self.interpLin(0.01, 0.045, 0.0005, 0.006, rho_s)
                                c2_PRS4_vl = self.interpLin(0.012, 0.06, 0.0005, 0.006, rho_s)     
                                c2_PRS2_vg = self.interpLin(0.004, 0.005, 0.0005, 0.006, rho_s)
                                c2_PRS3_vg = self.interpLin(0.005, 0.045, 0.0005, 0.006, rho_s)
                                c2_PRS4_vg = self.interpLin(0.006, 0.06, 0.0005, 0.006, rho_s)
                            if rho_t >= 0.006:
                                c2_PRT1_vl, c2_PRT2_vl, c2_PRT3_vl, c2_PRT4_vl = 0, 0.005, 0.045, 0.06
                                c2_PRT1_vg, c2_PRT2_vg, c2_PRT3_vg, c2_PRT4_vg = 0, 0.005, 0.045, 0.06
                            elif rho_t <= 0.0005:
                                c2_PRT1_vl, c2_PRT2_vl, c2_PRT3_vl, c2_PRT4_vl = 0, 0.005, 0.01, 0.012
                                c2_PRT1_vg, c2_PRT2_vg, c2_PRT3_vg, c2_PRT4_vg = 0, 0.004, 0.005, 0.006
                            else:
                                c2_PRT2_vl = 0.005
                                c2_PRT3_vl = self.interpLin(0.01, 0.045, 0.0005, 0.006, rho_t)
                                c2_PRT4_vl = self.interpLin(0.012, 0.06, 0.0005, 0.006, rho_t)     
                                c2_PRT2_vg = self.interpLin(0.004, 0.005, 0.0005, 0.006, rho_t)
                                c2_PRT3_vg = self.interpLin(0.005, 0.045, 0.0005, 0.006, rho_t)
                                c2_PRT4_vg = self.interpLin(0.006, 0.06, 0.0005, 0.006, rho_t)
                        if paf >= 0.6:
                            if rho_s >= 0.006:
                                c2_PRS1_vl, c2_PRS2_vl, c2_PRS3_vl, c2_PRS4_vl = 0, 0.003, 0.009, 0.01
                                c2_PRS1_vg, c2_PRS2_vg, c2_PRS3_vg, c2_PRS4_vg = 0, 0.003, 0.007, 0.008
                            elif rho_s <= 0.0005:
                                c2_PRS1_vl, c2_PRS2_vl, c2_PRS3_vl, c2_PRS4_vl = 0, 0.002, 0.003, 0.004
                                c2_PRS1_vg, c2_PRS2_vg, c2_PRS3_vg, c2_PRS4_vg = 0, 0.000001, 0.000001, 0.000001
                            else:
                                c2_PRS2_vl = self.interpLin(0.002, 0.003, 0.0005, 0.006, rho_s)
                                c2_PRS3_vl = self.interpLin(0.003, 0.009, 0.0005, 0.006, rho_s)
                                c2_PRS4_vl = self.interpLin(0.004, 0.01, 0.0005, 0.006, rho_s) 
                                c2_PRS2_vg = self.interpLin(0.000001, 0.003, 0.0005, 0.006, rho_s)
                                c2_PRS3_vg = self.interpLin(0.000001, 0.007, 0.0005, 0.006, rho_s)
                                c2_PRS4_vg = self.interpLin(0.000001, 0.008, 0.0005, 0.006, rho_s)   
                            if rho_t >= 0.006:
                                c2_PRT1_vl, c2_PRT2_vl, c2_PRT3_vl, c2_PRT4_vl = 0, 0.003, 0.009, 0.01
                                c2_PRT1_vg, c2_PRT2_vg, c2_PRT3_vg, c2_PRT4_vg = 0, 0.003, 0.007, 0.008
                            elif rho_t <= 0.0005:
                                c2_PRT1_vl, c2_PRT2_vl, c2_PRT3_vl, c2_PRT4_vl = 0, 0.002, 0.003, 0.004
                                c2_PRT1_vg, c2_PRT2_vg, c2_PRT3_vg, c2_PRT4_vg = 0, 0.000001, 0.000001, 0.000001
                            else:
                                c2_PRT2_vl = self.interpLin(0.002, 0.003, 0.0005, 0.006, rho_t)
                                c2_PRT3_vl = self.interpLin(0.003, 0.009, 0.0005, 0.006, rho_t)
                                c2_PRT4_vl = self.interpLin(0.004, 0.01, 0.0005, 0.006, rho_t) 
                                c2_PRT2_vg = self.interpLin(0.000001, 0.003, 0.0005, 0.006, rho_t)
                                c2_PRT3_vg = self.interpLin(0.000001, 0.007, 0.0005, 0.006, rho_t)
                                c2_PRT4_vg = self.interpLin(0.000001, 0.008, 0.0005, 0.006, rho_t)
                        else:
                            c2_PRS2_g_vg = self.interpLin(0.003, 0.005, 0.6, 0.1, paf)
                            c2_PRS3_g_vg = self.interpLin(0.007, 0.045, 0.6, 0.1, paf)
                            c2_PRS4_g_vg = self.interpLin(0.008, 0.06, 0.6, 0.1, paf)
                            c2_PRS2_g_vl = self.interpLin(0.003, 0.005, 0.6, 0.1, paf)
                            c2_PRS3_g_vl = self.interpLin(0.009, 0.045, 0.6, 0.1, paf)
                            c2_PRS4_g_vl = self.interpLin(0.01, 0.06, 0.6, 0.1, paf)
                            c2_PRS2_l_vg = self.interpLin(0.000001, 0.004, 0.6, 0.1, paf)
                            c2_PRS3_l_vg = self.interpLin(0.000001, 0.005, 0.6, 0.1, paf)
                            c2_PRS4_l_vg = self.interpLin(0.000001, 0.006, 0.6, 0.1, paf)
                            c2_PRS2_l_vl = self.interpLin(0.002, 0.005, 0.6, 0.1, paf)
                            c2_PRS3_l_vl = self.interpLin(0.003, 0.01, 0.6, 0.1, paf)
                            c2_PRS4_l_vl = self.interpLin(0.004, 0.012, 0.6, 0.1, paf)                            
                            c2_PRT2_g_vg = self.interpLin(0.003, 0.005, 0.6, 0.1, paf)
                            c2_PRT3_g_vg = self.interpLin(0.007, 0.045, 0.6, 0.1, paf)
                            c2_PRT4_g_vg = self.interpLin(0.008, 0.06, 0.6, 0.1, paf)
                            c2_PRT2_g_vl = self.interpLin(0.003, 0.005, 0.6, 0.1, paf)
                            c2_PRT3_g_vl = self.interpLin(0.009, 0.045, 0.6, 0.1, paf)
                            c2_PRT4_g_vl = self.interpLin(0.01, 0.06, 0.6, 0.1, paf)
                            c2_PRT2_l_vg = self.interpLin(0.000001, 0.004, 0.6, 0.1, paf)
                            c2_PRT3_l_vg = self.interpLin(0.000001, 0.005, 0.6, 0.1, paf)
                            c2_PRT4_l_vg = self.interpLin(0.000001, 0.006, 0.6, 0.1, paf)
                            c2_PRT2_l_vl = self.interpLin(0.002, 0.005, 0.6, 0.1, paf)
                            c2_PRT3_l_vl = self.interpLin(0.003, 0.01, 0.6, 0.1, paf)
                            c2_PRT4_l_vl = self.interpLin(0.004, 0.012, 0.6, 0.1, paf)
                            if rho_s >= 0.006:
                                c2_PRS2_vg = c2_PRS2_g_vg
                                c2_PRS3_vg = c2_PRS3_g_vg
                                c2_PRS4_vg = c2_PRS4_g_vg
                                c2_PRS2_vl = c2_PRS2_g_vl
                                c2_PRS3_vl = c2_PRS3_g_vl
                                c2_PRS4_vl = c2_PRS4_g_vl
                            elif rho_s <= 0.0005:
                                c2_PRS2_vg = c2_PRS2_l_vg
                                c2_PRS3_vg = c2_PRS3_l_vg
                                c2_PRS4_vg = c2_PRS4_l_vg
                                c2_PRS2_vl = c2_PRS2_l_vl
                                c2_PRS3_vl = c2_PRS3_l_vl
                                c2_PRS4_vl = c2_PRS4_l_vl
                            else:
                                c2_PRS2_vg = self.interpLin(c2_PRS2_l_vg, c2_PRS2_g_vg, 0.0005, 0.006, rho_s)
                                c2_PRS3_vg = self.interpLin(c2_PRS3_l_vg, c2_PRS3_g_vg, 0.0005, 0.006, rho_s)
                                c2_PRS4_vg = self.interpLin(c2_PRS4_l_vg, c2_PRS4_g_vg, 0.0005, 0.006, rho_s)   
                                c2_PRS2_vl = self.interpLin(c2_PRS2_l_vl, c2_PRS2_g_vl, 0.0005, 0.006, rho_s)
                                c2_PRS3_vl = self.interpLin(c2_PRS3_l_vl, c2_PRS3_g_vl, 0.0005, 0.006, rho_s)
                                c2_PRS4_vl = self.interpLin(c2_PRS4_l_vl, c2_PRS4_g_vl, 0.0005, 0.006, rho_s)
                            if rho_t >= 0.006:
                                c2_PRT2_vg = c2_PRT2_g_vg
                                c2_PRT3_vg = c2_PRT3_g_vg
                                c2_PRT4_vg = c2_PRT4_g_vg
                                c2_PRT2_vl = c2_PRT2_g_vl
                                c2_PRT3_vl = c2_PRT3_g_vl
                                c2_PRT4_vl = c2_PRT4_g_vl
                            elif rho_t <= 0.0005:
                                c2_PRT2_vg = c2_PRT2_l_vg
                                c2_PRT3_vg = c2_PRT3_l_vg
                                c2_PRT4_vg = c2_PRT4_l_vg
                                c2_PRT2_vl = c2_PRT2_l_vl
                                c2_PRT3_vl = c2_PRT3_l_vl
                                c2_PRT4_vl = c2_PRT4_l_vl
                            else:
                                c2_PRT2_vg = self.interpLin(c2_PRT2_l_vg, c2_PRT2_g_vg, 0.0005, 0.006, rho_t)
                                c2_PRT3_vg = self.interpLin(c2_PRT3_l_vg, c2_PRT3_g_vg, 0.0005, 0.006, rho_t)
                                c2_PRT4_vg = self.interpLin(c2_PRT4_l_vg, c2_PRT4_g_vg, 0.0005, 0.006, rho_t)   
                                c2_PRT2_vl = self.interpLin(c2_PRT2_l_vl, c2_PRT2_g_vl, 0.0005, 0.006, rho_t)
                                c2_PRT3_vl = self.interpLin(c2_PRT3_l_vl, c2_PRT3_g_vl, 0.0005, 0.006, rho_t)
                                c2_PRT4_vl = self.interpLin(c2_PRT4_l_vl, c2_PRT4_g_vl, 0.0005, 0.006, rho_t)
                        if vC_s >= 0.5:
                            c2_PRS2 = c2_PRS2_vg
                            c2_PRS3 = c2_PRS3_vg
                            c2_PRS4 = c2_PRS4_vg
                        elif vC_s <= 0.25:
                            c2_PRS2 = c2_PRS2_vl
                            c2_PRS3 = c2_PRS3_vl
                            c2_PRS4 = c2_PRS4_vl
                        else:
                            c2_PRS2 = self.interpLin(c2_PRS2_vl, c2_PRS2_vg, 0.25, 0.5, vC_s)
                            c2_PRS3 = self.interpLin(c2_PRS3_vl, c2_PRS3_vg, 0.25, 0.5, vC_s)
                            c2_PRS4 = self.interpLin(c2_PRS4_vl, c2_PRS4_vg, 0.25, 0.5, vC_s)
                        if vC_t >= 0.5:
                            c2_PRT2 = c2_PRT2_vg
                            c2_PRT3 = c2_PRT3_vg
                            c2_PRT4 = c2_PRT4_vg
                        elif vC_t <= 0.25:
                            c2_PRT2 = c2_PRT2_vl
                            c2_PRT3 = c2_PRT3_vl
                            c2_PRT4 = c2_PRT4_vl
                        else:
                            c2_PRT2 = self.interpLin(c2_PRT2_vl, c2_PRT2_vg, 0.25, 0.5, vC_t)
                            c2_PRT3 = self.interpLin(c2_PRT3_vl, c2_PRT3_vg, 0.25, 0.5, vC_t)
                            c2_PRT4 = self.interpLin(c2_PRT4_vl, c2_PRT4_vg, 0.25, 0.5, vC_t)
                            
                    if any("3" in i for i in self.c_condition) or any("4" in i for i in self.c_condition):
                        c3_PRS1 = 0
                        c3_PRT1 = 0
                        if paf <= 0.1:
                            if rho_s >= 0.006:
                                c3_PRS1, c3_PRS2, c3_PRS3, c3_PRS4 = 0, 0.000001, 0.045, 0.06
                            elif rho_s <= 0.0005:
                                c3_PRS1, c3_PRS2, c3_PRS3, c3_PRS4 = 0, 0.000001, 0.005, 0.006
                            else:
                                c3_PRS2 = 0.000001
                                c3_PRS3 = self.interpLin(0.005, 0.045, 0.0005, 0.006, rho_s)
                                c3_PRS4 = self.interpLin(0.006, 0.06, 0.0005, 0.006, rho_s)
                            if rho_t >= 0.006:
                                c3_PRT1, c3_PRT2, c3_PRT3, c3_PRT4 = 0, 0.000001, 0.045, 0.06                               
                            elif rho_t <= 0.0005:
                                c3_PRT1, c3_PRT2, c3_PRT3, c3_PRT4 = 0, 0.000001, 0.005, 0.006
                            else:
                                c3_PRT2 = 0.000001
                                c3_PRT3 = self.interpLin(0.005, 0.045, 0.0005, 0.006, rho_t)
                                c3_PRT4 = self.interpLin(0.006, 0.06, 0.0005, 0.006, rho_t)
                        if paf >= 0.6:
                            if rho_s >= 0.006:
                                c3_PRS1, c3_PRS2, c3_PRS3, c3_PRS4 = 0, 0.000001, 0.007, 0.008
                            elif rho_s <= 0.0005:
                                c3_PRS1, c3_PRS2, c3_PRS3, c3_PRS4 = 0, 0.000001, 0.000001, 0.000001
                            else:
                                c3_PRS2 = 0.000001
                                c3_PRS3 = self.interpLin(0.000001, 0.007, 0.0005, 0.006, rho_s)
                                c3_PRS4 = self.interpLin(0.000001, 0.008, 0.0005, 0.006, rho_s)
                            if rho_t >= 0.006:
                                c3_PRT1, c3_PRT2, c3_PRT3, c3_PRT4 = 0, 0.000001, 0.007, 0.008
                            elif rho_t <= 0.0005:
                                c3_PRT1, c3_PRT2, c3_PRT3, c3_PRT4 = 0, 0.000001, 0.000001, 0.000001
                            else:
                                c3_PRT2 = 0.000001
                                c3_PRT3 = self.interpLin(0.000001, 0.007, 0.0005, 0.006, rho_t)
                                c3_PRT4 = self.interpLin(0.000001, 0.008, 0.0005, 0.006, rho_t)
                        else:
                            c3_PRS2 = 0.000001
                            c3_PRS3_g = self.interpLin(0.007, 0.045, 0.6, 0.1, paf)
                            c3_PRS4_g = self.interpLin(0.008, 0.06, 0.6, 0.1, paf)
                            c3_PRS3_l = self.interpLin(0.000001, 0.005, 0.6, 0.1, paf)
                            c3_PRS4_l = self.interpLin(0.000001, 0.006, 0.6, 0.1, paf)
                            c3_PRT2 = 0.000001
                            c3_PRT3_g = self.interpLin(0.007, 0.045, 0.6, 0.1, paf)
                            c3_PRT4_g = self.interpLin(0.008, 0.06, 0.6, 0.1, paf)
                            c3_PRT3_l = self.interpLin(0.000001, 0.005, 0.6, 0.1, paf)
                            c3_PRT4_l = self.interpLin(0.000001, 0.006, 0.6, 0.1, paf)
                            if rho_s >= 0.006:
                                c3_PRS3 = c3_PRS3_g
                                c3_PRS4 = c3_PRS4_g
                            elif rho_s <= 0.0005:
                                c3_PRS3 = c3_PRS3_l
                                c3_PRS4 = c3_PRS4_l
                            else:
                                c3_PRS3 = self.interpLin(c3_PRS3_l, c3_PRS3_g, 0.0005, 0.006, rho_s)
                                c3_PRS4 = self.interpLin(c3_PRS4_l, c3_PRS4_g, 0.0005, 0.006, rho_s)
                            if rho_t >= 0.006:
                                c3_PRT3 = c3_PRT3_g
                                c3_PRT4 = c3_PRT4_g
                            elif rho_t <= 0.0005:
                                c3_PRT3 = c3_PRT3_l
                                c3_PRT4 = c3_PRT4_l
                            else:
                                c3_PRT3 = self.interpLin(c3_PRT3_l, c3_PRT3_g, 0.0005, 0.006, rho_t)
                                c3_PRT4 = self.interpLin(c3_PRT4_l, c3_PRT4_g, 0.0005, 0.006, rho_t)
                        c4_PRS1 = c3_PRS1
                        c4_PRS2 = c3_PRS2
                        c4_PRS3 = c3_PRS3
                        c4_PRS4 = c3_PRS4
                        c4_PRT1 = c3_PRT1
                        c4_PRT2 = c3_PRT2
                        c4_PRT3 = c3_PRT3
                        c4_PRT4 = c3_PRT4
                        
                    # Specify dummy values for conditions that do not control
                    if not any("1" in i for i in self.c_condition):
                        c1_PRS1, c1_PRS2, c1_PRS3, c1_PRS4 = 1, 1, 1, 1
                        c1_PRT1, c1_PRT2, c1_PRT3, c1_PRT4 = 1, 1, 1, 1
                    if not any("2" in i for i in self.c_condition):
                        c2_PRS1, c2_PRS2, c2_PRS3, c2_PRS4 = 1, 1, 1, 1
                        c2_PRT1, c2_PRT2, c2_PRT3, c2_PRT4 = 1, 1, 1, 1
                    if not any("3" in i for i in self.c_condition):
                        c3_PRS1, c3_PRS2, c3_PRS3, c3_PRS4 = 1, 1, 1, 1
                        c3_PRT1, c3_PRT2, c3_PRT3, c3_PRT4 = 1, 1, 1, 1
                    if not any("4" in i for i in self.c_condition):
                        c4_PRS1, c4_PRS2, c4_PRS3, c4_PRS4 = 1, 1, 1, 1
                        c4_PRT1, c4_PRT2, c4_PRT3, c4_PRT4 = 1, 1, 1, 1

                    # Take the minimum outcomes among all specified conditions
                    PRS1, PRS2, PRS3, PRS4 = [
                        min(
                            c1_PRS1, c2_PRS1, c3_PRS1, c4_PRS1), min(
                            c1_PRS2, c2_PRS2, c3_PRS2, c4_PRS2), min(
                            c1_PRS3, c2_PRS3, c3_PRS3, c4_PRS3), min(
                            c1_PRS4, c2_PRS4, c3_PRS4, c4_PRS4)]
                    PRT1, PRT2, PRT3, PRT4 = [
                        min(
                            c1_PRT1, c2_PRT1, c3_PRT1, c4_PRT1), min(
                            c1_PRT2, c2_PRT2, c3_PRT2, c4_PRT2), min(
                            c1_PRT3, c2_PRT3, c3_PRT3, c4_PRT3), min(
                            c1_PRT4, c2_PRT4, c3_PRT4, c4_PRT4)]
                        
                else:
                    # Determine values required to calculate table constants 
                    # for ASCE 41-17 Tables 10-8 and 10-9 
                    # Note: Table 10-9 is the same for ASCE 41-17 and 41-13
                    
                    # Note e and general requirement
                    rho_t_s = min(max(self.Aswx / self.AgShearcx, 0.0005), 0.0175)
                    # Note e and general requirement
                    rho_t_t = min(max(self.Aswy / self.AgShearcy, 0.0005), 0.0175)
                    ss_ratio_s = max(0.2, self.ss_ratio_s)  # General requirement
                    ss_ratio_t = max(0.2, self.ss_ratio_t)  # General requirement
                    Naf_s = max((self.Ned * 1000) / (self.Ancx * self.fck),
                                0.1)  # Note b requirement
                    Naf_t = max((self.Ned * 1000) / (self.Ancy * self.fck),
                                0.1)  # Note b requirement
                    rho_l_s = self.Ascx / self.Agcx
                    rho_l_t = self.Ascy / self.Agcy
    
                    if any("3" in i for i in self.c_condition):
                        # Columns controlled by inadequate development or
                        # splicing along the clear height
    
                        # Compute constant "a"
                        if (self.shape == 'Rectangular Column') |\
                                (self.shape == 'Cruciform'):
                            a_s = max(
                                0.042 -
                                0.043 *
                                Naf_s +
                                0.63 *
                                rho_t_s -
                                0.023 *
                                ss_ratio_s,
                                0)  # Local s-direction
                            a_t = max(
                                0.042 -
                                0.043 *
                                Naf_t +
                                0.63 *
                                rho_t_t -
                                0.023 *
                                ss_ratio_t,
                                0)  # Local t-direction
                            b_s = max(a_s, 0.5 / (5 + Naf_s * self.fck /
                                                  (0.8 * rho_t_s * self.fywd)) - 0.01)
                            b_t = max(a_t, 0.5 / (5 + Naf_t * self.fck /
                                                  (0.8 * rho_t_t * self.fywd)) - 0.01)
                        elif self.shape == 'Circular':
                            a_s = max(
                                0.06 -
                                0.06 *
                                Naf_s +
                                1.3 *
                                rho_t_s -
                                0.037 *
                                ss_ratio_s,
                                0)  # Local s-direction
                            a_t = max(
                                0.06 -
                                0.06 *
                                Naf_t +
                                1.3 *
                                rho_t_t -
                                0.037 *
                                ss_ratio_t,
                                0)  # Local t-direction
                            b_s = max(a_s, 0.65 / (5 + Naf_s * self.fck /
                                                   (0.8 * rho_t_s * self.fywd)) - 0.01)
                            b_t = max(a_t, 0.65 / (5 + Naf_t * self.fck /
                                                   (0.8 * rho_t_t * self.fywd)) - 0.01)
    
                        if Naf_s > 0.5:  # Note a requirement
                            b_s5 = max(a_s, 0.5 / (5 + 0.5 * self.fck /
                                                   (0.8 * rho_t_s * self.fywd)) - 0.01)
                            b_s7 = max(a_s, 0.5 / (5 + 0.7 * self.fck /
                                                   (0.8 * rho_t_s * self.fywd)) - 0.01)
                            b_s = self.interpLin(b_s5, b_s7, 0.5, 0.7, Naf_s)
                        if Naf_t > 0.5:  # Note a requirement
                            b_t5 = max(a_t, 0.5 / (5 + 0.5 * self.fck /
                                                   (0.8 * rho_t_t * self.fywd)) - 0.01)
                            b_t7 = max(a_t, 0.5 / (5 + 0.7 * self.fck /
                                                   (0.8 * rho_t_t * self.fywd)) - 0.01)
                            b_t = self.interpLin(b_t5, b_t7, 0.5, 0.7, Naf_t)
    
                        # Compute constant "c"
                        c_s = max(0.24 - 0.4 * Naf_s, 0)
                        c_t = max(0.24 - 0.4 * Naf_t, 0)
    
                        # Determine rotation limits
                        PRS1, PRS2, PRS3, PRS4 = 0, min(
                            0.005, 0.15 * a_s), 0.5 * b_s, 0.7 * b_s
                        PRT1, PRT2, PRT3, PRT4 = 0, min(
                            0.005, 0.15 * a_t), 0.5 * b_t, 0.7 * b_t
    
                    else:
                        # Columns not controlled by inadequate development or
                        # splicing along the clear height
                        
                        # Compute constant "a"
                        a_s = min(max(0, (rho_t_s * self.fywd) /
                                      (8 * rho_l_s * self.fyk)), 0.025)
                        a_t = min(max(0, (rho_t_t * self.fywd) /
                                      (8 * rho_l_t * self.fyk)), 0.025)
                        # Compute constant "b"
                        # Note e requirement
                        b_s = min(max(0, 0.012 - 0.085 * Naf_s + 12 *
                                      min(rho_t_s, 0.0075), a_s), 0.06)
                        # Note e requirement
                        b_t = min(max(0, 0.012 - 0.085 * Naf_t + 12 *
                                      min(rho_t_t, 0.0075), a_t), 0.06)
    
                        # Compute constant "c"
                        c_s = min(0.15 + 36 * rho_t_s, 0.4)
                        c_t = min(0.15 + 36 * rho_t_t, 0.4)
    
                        # Determine rotation limits (must be in ascending order)
                        PRS1, PRS2, PRS3, PRS4 = 0, 0.1 * b_s, 0.5 * b_s, 0.7 * b_s
                        PRT1, PRT2, PRT3, PRT4 = 0, 0.1 * b_t, 0.5 * b_t, 0.7 * b_t

        else:
            PRS1, PRS2, PRS3, PRS4 = 0, 0, 0, 0
            PRT1, PRT2, PRT3, PRT4 = 0, 0, 0, 0

        PRS = [PRS1, PRS2, PRS3, PRS4]
        PRT = [PRT1, PRT2, PRT3, PRT4]
        PRS = [round(i, 6) for i in PRS]
        PRT = [round(i, 6) for i in PRT]

        return PRS



# -*- coding: utf-8 -*-
"""
Created on Thu Apr 26 12:54:15 2018

@author: kevin.stanton

This class initializes an XTRACT model given user defined data from xtract3.py
"""


class beamGeometry():

    def __sect__(self, aData):

        # Write section builder properties for circular section
        if aData.section_type == 'Circular':
            section_area = 3.14 * pow(float(aData.diameter_circ) / 2000, 2.0)

        # Write section builder properties for rectangular beam section
        elif aData.section_type == 'Rectangular Beam':
            section_area = float(aData.rect_width) / 1000 * \
                float(aData.rect_height) / 1000

        # Write section builder properties for rectangular column section
        elif aData.section_type == 'Rectangular Column':
            section_area = round(
                (float(
                    aData.rect_width) /
                    1000) *
                float(
                    aData.rect_height) /
                1000,
                5)

        # Write section builder properties for T Beam section
        elif aData.section_type == 'T Beam':
            section_area = (float(aData.f_width) / 1000) * float(aData.f_thick) / 1000 + float(
                aData.t_width) / 1000 * (float(aData.t_height) / 1000 - float(aData.f_thick) / 1000)

        # Write section builder properties for Cruciform section
        elif aData.section_type == 'Cruciform':
            section_area = round(((float(aData.tX) *
                                   float(aData.bY) +
                                   float(aData.tY) *
                                   float(aData.bX) -
                                   float(aData.tX) *
                                   float(aData.tY)) /
                                  (1000**2)), 5)
        return section_area

    def runElastic(self, aData):

        # Calculate required geometric data
        section_area = self.__sect__(aData)

        return section_area



# -*- coding: utf-8 -*-
"""
Created on Thu Apr 18 11:53:35 2019

@author: kevin.stanton

This script contains functions for the autodyna toolkit.
"""

# Import shared libraries
import os
import sys
import pandas as pd
import numpy as np
import shutil
import time

# Import project libraries
from .util import util
from .construct_hysBeams import info_hysBeams
from .iXTRACT_envelope import iXTRACT_envelope
from .Shear_Calcs import Shear_Calcs
from .Autorun import Autorun
from .Mat_Hys_Beam import Mat_Hys_Beam
from .settings import settings
from .construct_elasticBeams import info_elasticBeams
from .Elastic_Beam import Elastic_Beam
from .beamGeometry import beamGeometry


def MAT_HYSTERETIC_BEAM(
        batchMode=False,
        iSet=1,
        pInputs=os.path.join(os.getcwd(),'inputs_hysBeams.xlsx'),
        pSave=os.path.join(os.getcwd(),'Output_hysBeams'),
        codeOption='EC2',
        concreteFactor='normal',
        fParams='ASCE 41-17',
        tbOption='default',
        concrete_strain_limits=[
            0.0012,
            0.0024,
            0.0035],
        steel_strain_limits=[
            1.0,
            1.5],
        npoints=401,
        mphisolution='BiSection',
        bilinearflag=False,
        cLimits='explicit',
        cleanCurves=True,
        skipFlag=True,
        pXTRACT=r'C:\Program Files (x86)\TRC\XTRACT\XTRACT.exe',
        mInter=False,
        timingControl='automatic',
        controlMode='default',
        author='user',
        company='Arup',
        job_name='dev',
        showPlots=True,
        write_log=False,
        user_xpj=False,
        settings_path=''):
    """
    This function writes MAT_HYSTERETIC_BEAM cards for LS-DYNA. DO NOT TOUCH
    THE MOUSE WHILE CODE IS RUNNING unless the manual intervention option is
    enabled.

    @author: kevin.stanton

    Parameters
    ----------
    batchMode : bool
        Batch mode option
        Default = False -> results only returned for the given iSet
    iSet : int
        Input set for a single analysis (ignored if batchMode = 'on')
    pInputs : str
        Input file path (.txt, .csv, or .xlsx)
        Default = cwd\inputs_hysBeams.xlsx
        Extra values/blank lines are automatically ignored
    pSave : str
        Option to specify a different file path to write results
        Default = cwd\Output_hysBeams
        Specified folder will be tagged with current date/time
    codeOption : str
        'EC2' for Eurocode 2 [default]
        'CSA' for Canadian Standards Association (CSA A23.3-14)
    concreteFactor : str
        Factor to account for lightweight aggregate concrete (or not)
        Only applicable to CSA (known as parameter lambda)
        'normal' (lamda=1) [default]; 'semi-light' (lambda=0.85); or 'light'
        (lambda=0.75)
    fParams : str or bool
        Options to compute/write plastic rotation thresholds
        Valid arguments = 'ASCE 41-17' (default), 'ASCE 41-13', or False
        If False, no element deletion will take place
        PRSi/PRTi = 0 if required input data is missing
    tbOption : str
        Treatment of T-Beams ('default' or 'in-slab')
        'in-slab' -> density_reduced = density*(Area_eff/Area_total)
    concrete_strain_limits : list of floats
        Limiting concrete strains for enveloping PM curve
        For more refined analyses use:
            [0.0012,0.0015,0.0018,0.0021,0.0024,0.0027,0.003,0.0033,0.0035]
        ASCE 41-17, Section 10.3.3.1 [Usable Strain Limits]:
            [0.002,0.003]
    steel_strain_limits : list of floats
        Limiting steel strains for enveloping PM curve
        Defined as: (yield stress / Young's modulus) x steel_strain_limits[i]
        default = [1.0, 1.5, 2.0]
    npoints : int
        Number of points desired on the final, enveloped PM curve
        Default = 401
    mphisolution : str
        Solution method for moment curvature plot
        default = 'BiSection'
        'MUDN' -> iteration at Minimum Unbalanced Displacement Norm
    bilinearflag : bool
        Flag to use bi-linearized moment curvature data in LS-DYNA
        Default = False
    cLimits : str
        Yield moment interpretation ('strict', 'flex', 'explicit')
        'strict' and 'flex' are ML methods
        'explicit' (default) is based on checking developed strains against
        user-defined limits
    cleanCurves:
        Flag to write smoothed moment curvature data to LS-DYNA
        default = True
    skipFlag : True
        Flag to skip XTRACT analysis if appropriate data already exists
        Relevant data must be saved in working directory ([ID].txt files)
        Default = True
    pXTRACT : str
        XTRACT path
        default = r'C:\Program Files (x86)\TRC\XTRACT\XTRACT.exe'
        The code automatically checks the os for XTRACT but sometimes
        environment settings force the path to be defined explicitely
    mIter : bool
        Option to enable manual intervention mode for XTRACT analysis
        Default = False
    timingControl : str
        Toggle for automatic (CPU-based) analysis timing or fixed timing.
        Acceptable arguments:
            'automatic' -> CPU usage based analysis timing [default]
            'fixed' -> fixed analysis time from 'settings.py'
    controlMode : str
        Mouse movement control options ('default' or 'mod1')
        If 'default' causes an error, try 'mod1'
    author : str
        Name of author
        Default = 'user'
    company : str
        Company name
        Default = 'Arup'
    job_name : str
        Project name
        Default = 'dev'
    showPlots : bool
        Show XTRACT plots when running section analysis (True or False)
        May run faster if 'False'
        Default = True
    write_log : bool
        Option to write the contents of the terminal to a log file
        Default = False
    user_xpj : bool
        Option to use a user-defined .xpj to run XTRACT
        File name must match info_filename for given iSet and file must be 
        stored in the cwd
    settings_path : str
        Path to settings file for user-defined mouse click positions
        Default setting used if unspecified

    Returns
    -------
    Beam-RC_Beam_[ID].key : file(s)
        LS-DYNA keyword file is written for each beam instance
        Includes section and material card data
    autobeams.key : file
        LS-DYNA keyword containing all batch results in one file
    [ID].txt : file
        A text file containing the raw XTRACT output is saved for each
        beam instance
    M_phi_xx_[ID].png, M_phi_yy_[ID].png : files
        Images of the interpretted moment-curvature plots for both loading
        directions are saved each beam instance
    Moment_Curvature_[ID].csv : file
        Processed moment curvature data for beam instance [ID]
    Moment_Curvature_RAW_[ID].csv : file
        Raw moment curvature data for beam instance [ID]
    Moment_Interaction_[ID].csv : file
        Axial load-moment interaction data for beam instance [ID]
        Applies to last entries in steel and concrete strain limit lists
    Moment_Interaction[ID].csv : file
        Axial load-moment interaction data for beam instance [ID]
        Includes all results for the different steel and concrete strain limits
    Moment_Interaction_Envelope[ID].csv : file
        Enveloped axial load-moment interaction data for beam instance [ID]
    Moment_Plastic_Rotation_[ID].csv : file
        Moment-plastic rotation interaction data for beam instance [ID]
    PMxx[ID].csv, PMyy[ID].csv : file
        Enveloped axial load-moment interaction data for beam instance [ID]
    jsonData : str
        Contents of Beam-RC_Beam_[ID].key written in JSON format
    keyPath : str
        Path to generated file 'autobeams.key'
    hysBeams.log : file (optional)
        The console output written to a log file
        Not written in write_log = False
    """

    # Check that inputs file exists and handle if not
    if os.path.isfile(pInputs) == False:
        raise AssertionError(
            'Inputs file not found: ' + str(pInputs))

    # Parse input file
    # Get input file type
    ext = os.path.splitext(pInputs)[1]
    # Specify interpreter order
    iData_order = ['info_filename',
                   'Fc',
                   'Ft',
                   'ey',
                   'ecu',
                   'esp',
                   'ef',
                   'Ec',
                   'den',
                   'v',
                   'Fy',
                   'Fu',
                   'esh',
                   'esu',
                   'Es',
                   'section_type',
                   'cover',
                   'diameter_circ',
                   'bar_diam_circ',
                   'num_bars_circ',
                   'rect_width',
                   'rect_height',
                   'n_t',
                   'n_b',
                   'db_t',
                   'db_b',
                   'num_bars_col',
                   'bar_diam_col',
                   't_width',
                   't_height',
                   'f_width',
                   'f_thick',
                   'f_lay',
                   'f_bar',
                   'f_spac',
                   't_nt',
                   't_tbar',
                   't_nb',
                   't_bbar',
                   'bX',
                   'bY',
                   'tX',
                   'tY',
                   'cX',
                   'cY',
                   'cBar_centerD',
                   'cBar_vFlangeD',
                   'cBar_hFlangeD',
                   'cBar_vFlangeN',
                   'cBar_hFlangeN',
                   'cBar_rows',
                   'M_load',
                   'N_load',
                   'V_load',
                   'd_shear',
                   's_shear',
                   'n_legs',
                   'gc_mat',
                   'gs_mat',
                   'Fywd',
                   'phl_xx',
                   'phl_yy',
                   'rb_condition',
                   'c_condition',
                   'ss_ratio_s',
                   'ss_ratio_t']

    # Read data
    if ext == '.txt':
        iData = pd.read_csv(pInputs, dtype=str, header=None,
                            sep=',\s+', engine='python')
    elif ext == '.csv':
        iData = pd.read_csv(pInputs, dtype=str, header=None)
    elif ext == '.xlsx':
        iData = pd.read_excel(pInputs, dtype=str, header=None)
    # Remove whitespaces from strings
    iData = iData.applymap(lambda x: x.strip() if isinstance(x, str) else x)
    # Remove data rows not spacified in iData_order
    iData = iData[iData.iloc[:, 0].isin(iData_order)]
    # Remove second column if full input format detected
    if 'List' in iData.iloc[0, 1]:
        iData = iData.drop(columns=1)
    # Re-index
    iData = iData.set_index(0)
    # Sort according to iData_order
    iData = iData.loc[iData_order]
    # Re-index columns to match iRun handles
    iData.columns = pd.RangeIndex(0, len(iData.columns))

    # Tag output folder with current time
    timestr = time.strftime("%Y%m%d-%H%M%S")
    pSave = str(pSave + '_' + timestr)
    os.makedirs(pSave)

    # Append static inputs to data array
    XTRACT = 'on'
    devMode = 'on'
    envelopeflag = True  # use PM enveloping approach
    sData = [
        author,
        company,
        job_name,
        pXTRACT,
        mInter,
        batchMode,
        iSet,
        fParams,
        tbOption,
        devMode,
        cLimits,
        cleanCurves,
        showPlots,
        controlMode,
        codeOption,
        concreteFactor,
        timingControl,
        pSave,
        write_log,
        settings_path]
    data = iData, sData

    # Add pSave to path
    sys.path.insert(0, pSave)
    
    # Store output in a log file if needed
    if write_log:
        class Logger(object):
            def __init__(self):
                self.terminal = sys.stdout
                self.log = open(os.path.join(pSave,'hysBeams.log'), 'w')

            def write(self, message):
                self.terminal.write(message)
                self.log.write(message)  

            def flush(self):
                self.log.flush() 

            def close(self):
                self.log.close()                
        sys.stdout = Logger()

    # Run required analyses
    if batchMode:

        keyNames = []
        iRun = 0
        fRun = iData.shape[1]  # Total number of runs for print statements
        while iRun < iData.shape[1]:
            # Initialize analysis inputs
            aData = info_hysBeams(data, iRun)

            # Process mouse control settings
            mcSettings = settings(aData)

            # Print warning statement if 'BH' is not included in the ID in
            # batchData.csv
            util.getName(aData)

            if envelopeflag:

                iSet = iRun + 1
                print('\n--------------------')
                print('Beam ' + aData.file_name +
                      ' (' + str(iSet) + ' of ' + str(fRun) + ')')
                print('--------------------\n')
                print('   Section type: ' + str(aData.section_type) + '\n')

                if XTRACT == 'on':
                    if not user_xpj:
                        print('   Creating XTRACT input file... \n')
                        saveName = str(aData.file_name) + '.txt'
                        if skipFlag:
                            if os.path.isfile(saveName) == False:
                                # Create the section in XTRACT
                                section_area = iXTRACT_envelope(aData).write_EnvelopeXtract(
                                    aData, concrete_strain_limits, steel_strain_limits, mphisolution)
                                # Run XTRACT
                                Autorun(
                                    aData,
                                    mcSettings).autorun_xtract(
                                    aData,
                                    n_strainlim=len(concrete_strain_limits) *
                                    len(steel_strain_limits))
                            else:
                                shutil.copy(saveName, pSave)
                        else:
                            # Move results to current output folder
                            shutil.copy(saveName, pSave + '\\' +
                                        str(aData.file_name) + '.txt')
                            # Create the section in XTRACT
                            section_area = iXTRACT_envelope(aData).write_EnvelopeXtract(
                                aData, concrete_strain_limits, steel_strain_limits, mphisolution)
                            # Run XTRACT
                            Autorun(
                                aData,
                                mcSettings).autorun_xtract(
                                aData,
                                n_strainlim=len(concrete_strain_limits) *
                                len(steel_strain_limits))
                    else:
                        xpj_name = str(aData.file_name) + '.xpj'
                        if os.path.isfile(xpj_name) == False:
                            raise AssertionError('Cannot find user-defined .xpj file.')
                        else:
                            print('   User-defined .xpj file located. Attempting to analyze in XTRACT... ')
                            # Run XTRACT
                            Autorun(
                                aData,
                                mcSettings, xpj_name).autorun_xtract(
                                aData,
                                n_strainlim=len(concrete_strain_limits) *
                                len(steel_strain_limits))

                # Calculate the PM interaction envelope
                util.getPM(
                    aData,
                    len(concrete_strain_limits) *
                    len(steel_strain_limits),
                    npoints)

                # Shear calculations
                print('   Performing shear calculations... \n')
                shear = Shear_Calcs(aData)
                Ned = float(aData.N_load)
                Vrdx, Vrdy = shear.Shear_cap(Ned)
                PRS, PRT = shear.prLimits()

                # Create individual .key files
                print('   Writing keyword data...')
                card = Mat_Hys_Beam(
                    aData,
                    aData.section_area,
                    envelopeflag,
                    len(concrete_strain_limits) *
                    len(steel_strain_limits),
                    bilinearflag)
                json_data = card.write_mat(aData, PRS, PRT)

                # Populate list of .key file names
                keyNames.append(os.path.join(aData.pSave,
                                             util.getName(aData, '.key')))
			
			# Update counter
            iRun = iRun + 1

        # Combine .key data into one file (autoBeams.key)
        util.combineKeys(keyNames)
        # Move autoBeams.key to output directory
        shutil.move('autobeams.key', aData.pSave)

        # Report on completion
        print('-- Finished --')
        
        # Close logger
        if write_log:
            Logger().close()
            
        # Return keyword path if pSave specified, JSON data if not
        if pSave:
            keyPath = os.path.join(aData.pSave,'autobeams.key')
            return keyPath
        else:
            return json_data

    elif not batchMode:

        # Specify data set to run on
        iRun = iSet - 1

        # Initialize analysis inputs
        aData = info_hysBeams(data, iRun)

        # Process mouse control settings
        mcSettings = settings(aData)

        if envelopeflag:

            print('\n--------------------')
            print('Beam ' + aData.file_name)
            print('--------------------\n')
            print('   Section type: ' + str(aData.section_type) + '\n')

            if XTRACT == 'on':
                if not user_xpj:
                    print('   Creating XTRACT input file... \n')
                    saveName = os.getcwd() + '\\' + str(aData.file_name) + '.txt'
                    if skipFlag:
                        if os.path.isfile(saveName) == False:
                            # Create the section in XTRACT
                            section_area = iXTRACT_envelope(aData).write_EnvelopeXtract(
                                aData, concrete_strain_limits, steel_strain_limits, mphisolution)
                            # Run XTRACT
                            Autorun(
                                aData,
                                mcSettings).autorun_xtract(
                                aData,
                                n_strainlim=len(concrete_strain_limits) *
                                len(steel_strain_limits))
                        else:
                            shutil.copy(saveName, pSave)
                    else:
                        # Move results to current output folder
                        shutil.copy(saveName, pSave + '\\' +
                                    str(aData.file_name) + '.txt')
                        # Create the section in XTRACT
                        section_area = iXTRACT_envelope(aData).write_EnvelopeXtract(
                            aData, concrete_strain_limits, steel_strain_limits, mphisolution)
                        # Run XTRACT
                        Autorun(
                            aData,
                            mcSettings).autorun_xtract(
                            aData,
                            n_strainlim=len(concrete_strain_limits) *
                            len(steel_strain_limits))
                else:
                        xpj_name = str(aData.file_name) + '.xpj'
                        if os.path.isfile(xpj_name) == False:
                            raise AssertionError('Cannot find user-defined .xpj file.')
                        else:
                            print('   User-defined .xpj file located. Attempting to analyze in XTRACT... ')
                            # Run XTRACT
                            Autorun(
                                aData,
                                mcSettings, xpj_name).autorun_xtract(
                                aData,
                                n_strainlim=len(concrete_strain_limits) *
                                len(steel_strain_limits))

            # Calculate PM interaction envelope
            util.getPM(aData,
                       len(concrete_strain_limits) *
                       len(steel_strain_limits),
                       npoints)

            # Calculate shear capacity
            print('   Performing shear calculations... \n')
            shear = Shear_Calcs(aData)
            Ned = float(aData.N_load)
            Vrdx, Vrdy = shear.Shear_cap(Ned)
            PRS, PRT = shear.prLimits()

            # Create .key file
            print('   Writing keyword data...')
            card = Mat_Hys_Beam(
                aData,
                aData.section_area,
                envelopeflag,
                len(concrete_strain_limits) *
                len(steel_strain_limits),
                bilinearflag)
            json_data = card.write_mat(aData, PRS, PRT)

        # Report on completion
        print('-- Finished --')

        # Close logger
        if write_log:
            Logger().close()
            
        # Return keyword path if pSave specified, JSON data if not
        if pSave:
            keyPath = os.path.join(aData.pSave, util.getName(aData, '.key'))
            return keyPath
        else:
            return json_data


def MAT_ELASTIC(
        batchMode=False,
        iSet=1,
        pInputs=os.path.join(os.getcwd(), 'inputs_elasticBeams.xlsx'),
        pSave=os.path.join(os.getcwd(), 'Output_elasticBeams'),
        tbOption='default',
        author='user',
        company='Arup',
        job_name='dev',
        write_log=False):
    """
    This function writes MAT_ELASTIC and section cards for LS-DYNA beam
    elements.

    @author: kevin.stanton

    Parameters
    ----------
    batchMode : bool
        Batch mode option ('on' or 'off')
        Default = False -> results only returned for the given iSet
    iSet : int
        Input set for a single analysis (ignored if batchMode = 'on')
    pInputs : str
        Input file path (.txt, .csv, or .xlsx)
        Default = os.getcwd() + '\\inputs.xlsx'
        Extra values/blank lines are automatically ignored
        May alternatively be defined as a list of strings in lieu of all other
        arguments
    pSave : str
        Option to specify a different file path to write results
        Default = os.getcwd() + '\\Output_elasticBeams'
        Specified folder will be tagged with current date/time
    tbOption : str
        Treatment of T-Beams ('default' or 'in-slab')
        'in-slab' -> density_reduced = density*(Area_eff/Area_total)
    author : str
        Name of author
        Default = 'user'
    company : str
        Company name
        Default = 'Arup'
    job_name : str
        Project name
        Default = 'dev'
    write_log : bool
        Option to write the contents of the terminal to a log file
        Default = False

    Returns
    -------
    Beam-RC_Beam_[ID].key : file(s)
        An LS-DYNA keyword file is written to the output directory for each
        beam analyzed
        Includes section and material card data
    autobeams.key : file
        LS-DYNA keyword containing all batch results in one file is written to
        the output directory
    jsonData : str
        Contents of Beam-RC_Beam_[ID].key written in JSON format
    keyPath : str
        Path to generated file 'autobeams.key'
    elasticBeams.log : file (optional)
        The console output is written to a log file the pSave directory stamped
        with the analysis start date/time
        Not written if write_log = False
    """

    # Specify interpreter order
    iData_order = ['info_filename',
                   'Ec',
                   'den',
                   'v',
                   'section_type',
                   'cover',
                   'diameter_circ',
                   'rect_width',
                   'rect_height',
                   't_width',
                   't_height',
                   'f_width',
                   'f_thick',
                   'bX',
                   'bY',
                   'tX',
                   'tY',
                   'cX',
                   'cY']

    # Parse inputs and handle the case where pInputs is a list of strings
    if type(pInputs) == list:
        batchMode = False
        iSet = 1
        pSave = False
        tbOption = 'default'
        author = 'user'
        company = 'Arup'
        job_name = 'dev'
        iData = [iData_order, pInputs]
        iData = pd.DataFrame(np.transpose(np.array(iData)))
    else:
        # Check that inputs file exists and handle if not
        if os.path.isfile(pInputs) == False:
            raise AssertionError(
                'Inputs file not found: ' + str(pInputs))
        # Get input file type
        ext = os.path.splitext(pInputs)[1]
        # Read data
        if ext == '.txt':
            iData = pd.read_csv(
                pInputs,
                dtype=str,
                header=None,
                sep=r',\s+',
                engine='python')
        elif ext == '.csv':
            iData = pd.read_csv(pInputs, dtype=str, header=None)
        elif ext == '.xlsx':
            iData = pd.read_excel(pInputs, dtype=str, header=None)

    # Remove whitespaces from strings
    iData = iData.applymap(lambda x: x.strip() if isinstance(x, str) else x)
    # Remove data rows not spacified in iData_order
    iData = iData[iData.iloc[:, 0].isin(iData_order)]
    # Remove second column if full input format detected
    if 'List' in iData.iloc[0, 1]:
        iData = iData.drop(columns=1)
    # Re-index
    iData = iData.set_index(0)
    # Sort according to iData_order
    iData = iData.loc[iData_order]
    # Re-index columns to match iRun handles
    iData.columns = pd.RangeIndex(0, len(iData.columns))

    if not pSave == False:
        # Tag output folder with current time
        timestr = time.strftime("%Y%m%d-%H%M%S")
        pSave = pSave + '_' + timestr
        os.makedirs(pSave)
        # Add pSave to path
        sys.path.insert(0, pSave)
    
    # Store output in a log file if needed
    if write_log:
        class Logger(object):
            def __init__(self):
                self.terminal = sys.stdout
                self.log = open(os.path.join(pSave,'elasticBeams.log'), 'w')

            def write(self, message):
                self.terminal.write(message)
                self.log.write(message)  

            def flush(self):
                self.log.flush()  
                
            def close(self):
                self.log.close()
        sys.stdout = Logger()        

    # Append static inputs to data array
    sData = [
        author,
        company,
        job_name,
        batchMode,
        iSet,
        tbOption,
        pSave,
        write_log]
    data = iData, sData

    # Run required analyses
    if batchMode:

        keyNames = []
        iRun = 0
        while iRun < iData.shape[1]:

            # Initialize analysis inputs
            aData = info_elasticBeams(data, iRun)

            # Print warning statement if 'BH' is not included in the ID in
            # batchData.csv
            util.getName(aData)

            print('\n--------------')
            print('Beam ' + aData.file_name)
            print('-------------- \n')
            print('   Section type: ' + str(aData.section_type) + '\n')

            # Compute geometric section data
            section_area = beamGeometry().runElastic(aData)

            # Create individual .key files
            print('   Writing keyword data...')
            card = Elastic_Beam(aData)
            json_data = card.write_mat(aData)

            # Populate list of .key file names
            keyNames.append(os.path.join(aData.pSave,
                                         util.getName(aData, '.key')))

            # Update counter
            iRun = iRun + 1

        # Combine .key data into one file (autoBeams.key)
        util.combineKeys(keyNames)
        # Move autoBeams.key to output directory
        shutil.move('autobeams.key', aData.pSave)

        # Close logger
        if write_log:
            Logger().close()
            
        # Return keyword path if pSave specified, JSON data if not
        if pSave:
            keyPath = os.path.join(aData.pSave,'autobeams.key')
            return keyPath
        else:
            return json_data

    elif not batchMode:

        # Specify data set to run on
        iRun = iSet - 1

        # Initialize analysis inputs
        aData = info_elasticBeams(data, iRun)

        print('\n--------------')
        print('Beam ' + aData.file_name)
        print('-------------- \n')
        print('   Section type: ' + str(aData.section_type) + '\n')

        # Compute geometric section data
        section_area = beamGeometry().runElastic(aData)

        # Create .key file
        print('   Writing keyword data...')
        card = Elastic_Beam(aData)
        json_data = card.write_mat(aData)
        
        # Close logger
        if write_log:
            Logger().close()
            
        # Return keyword path if pSave specified, JSON data if not
        if pSave:
            keyPath = os.path.join(aData.pSave, util.getName(aData, '.key'))
            return keyPath
        else:
            return json_data
            



"""
Created on Thu Apr 26 13:01:57 2018

@author: kevin.stanton

This class provides helper functions for autodyna_ac
"""

import pandas as pd
import numpy as np
import math
import time
from scipy.interpolate import interp1d
import os
from tkinter import messagebox
import win32api
import win32con


class util:

    def confirm(aData):

        if aData.mInter:
            answer = messagebox.askyesno(
                "Breakpoint Reached", "Ready to continue?")
            if not answer:
                raise AssertionError("Process terminated by user.")

    def click(x, y, button='left'):

        win32api.SetCursorPos((x, y))

        if button == 'right':
            win32api.mouse_event(win32con.MOUSEEVENTF_RIGHTDOWN, x, y, 0, 0)
            win32api.mouse_event(win32con.MOUSEEVENTF_RIGHTUP, x, y, 0, 0)
        elif button == 'left':
            win32api.mouse_event(win32con.MOUSEEVENTF_LEFTDOWN, x, y, 0, 0)
            win32api.mouse_event(win32con.MOUSEEVENTF_LEFTUP, x, y, 0, 0)

    def move(x, y):

        win32api.SetCursorPos((x, y))

    def combineKeys(keyNames, outFile='autoBeams.key'):
        """
        This function combines .key files present in the current woking directory
        into one file named 'autoBeams.key'.

        Parameters
        ----------
            keyNames : list
                - file names or full file paths to be combined as a list of strings
                - list elements must represent existing .key files
                - example: ['file1.key', 'file2.key', ...]
            outFile : str
                - optional [default='autoBeams.key']
                - file name to write combined .key data

        Returns
        -------
            [outfile].key : file
                - .key with data combined from keyNames list
                - written to the current working directory
        """

        # Define list of terms to delete in order to facilitate the merge
        deleteList = ['*KEYWORD', '*END']
        # Write the raw combined file (temp)
        with open('temp.key', 'wb') as outfile:
            for i in keyNames:
                with open(i, 'rb') as infile:
                    outfile.write(infile.read())
        # Remove the required strings and write the final file
        with open('temp.key') as oldFile, open(outFile, 'w') as newFile:
            newFile.write('*KEYWORD\n')
            for line in oldFile:
                if not any(
                        badString in line for badString in deleteList) and line.strip():
                    newFile.write(line)
            newFile.write('*END')
        # Delete the temp file
        os.remove('temp.key')

    def cutOutlier(x, y):

        numCut = 0
        i = 0
        xMod = []
        yMod = []
        while i < len(y) - 1:
            if float(y[i]) < float(y[i + 1]):
                xMod.append(x[i])
                yMod.append(y[i])
            elif i > 9:
                xMod.append(x[i])
                yMod.append(y[i])
            else:
                numCut = numCut + 1
            i = i + 1

        return xMod, yMod, numCut

    def expYield(M_phi, aData):

        steelThreshold = float(aData.Fy) / float(aData.Es)
        print('         Concrete yield strain = ' + str(aData.ey))
        print('         Steel yield strain = ' + str(steelThreshold))

        # Read in data
        Mxx_raw, phi_xx_raw, cMin_xx, sMin_xx, sMax_xx = M_phi.loc[:, 'Mxx (N-m)'], \
            M_phi.loc[:, 'phi_xx (1/m)'], M_phi.loc[:, 'ConfinedMinStrain_xx'], \
            M_phi.loc[:, 'SteelMinStrain_xx'], M_phi.loc[:, 'SteelMaxStrain_xx']

        Myy_raw, phi_yy_raw, cMin_yy, sMin_yy, sMax_yy = M_phi.loc[:, 'Myy (N-m)'], \
            M_phi.loc[:, 'phi_yy (1/m)'], M_phi.loc[:, 'ConfinedMinStrain_yy'], \
            M_phi.loc[:, 'SteelMinStrain_yy'], M_phi.loc[:, 'SteelMaxStrain_yy']

        # Remove outliers
        phi_xx, Mxx, numCutX = util.cutOutlier(phi_xx_raw, Mxx_raw)
        phi_yy, Myy, numCutY = util.cutOutlier(phi_yy_raw, Myy_raw)

        # Find coordinate of My
        # xx direction
        i = 1
        while i <= len(Mxx_raw):
            index = 0
            if abs(float(cMin_xx[i])) > float(aData.ey):
                print('         Governing yield - xx = confined concrete (XTRACT min)')
                index = min(max(i - 1, 4), 10)
                i = len(Mxx_raw)
            elif abs(float(sMax_xx[i])) > float(aData.Fy) / float(aData.Es):
                print('         Governing yield - xx = steel (XTRACT max)')
                index = min(max(i - 1, 4), 10)
                i = len(Mxx_raw)
            elif abs(float(sMin_xx[i])) > float(aData.Fy) / float(aData.Es):
                print('         Governing yield - xx = steel (XTRACT min)')
                index = min(max(i - 1, 4), 10)
                i = len(Mxx_raw)
            i = i + 1

        # Check that an appropriate index was found
        if index == 0:
            raise AssertionError(
                'Yield moment cannot be interpreted! Moment curvature must be computed for larger plastic rotations.')
        print('         index_xx =' + str(index))

        ind_xx = index
        xP_xx = float(phi_xx_raw[index])
        yP_xx = float(Mxx_raw[index])

        # yy direction
        i = 1
        while i <= len(Myy_raw):
            index = 0
            if abs(float(cMin_yy[i])) > float(aData.ey):
                print('         Governing yield - yy = confined concrete (XTRACT min)')
                index = min(max(i - 1, 4), 10)
                i = len(Myy_raw)
            elif abs(float(sMax_yy[i])) > float(aData.Fy) / float(aData.Es):
                print('         Governing yield - yy = steel (XTRACT max)')
                index = min(max(i - 1, 4), 10)
                i = len(Myy_raw)
            elif abs(float(sMin_yy[i])) > float(aData.Fy) / float(aData.Es):
                print('         Governing yield - yy = steel (XTRACT min)')
                index = min(max(i - 1, 4), 10)
                i = len(Myy_raw)
            i = i + 1

        # Check that an appropriate index was found
        if index == 0:
            raise AssertionError(
                'Yield moment cannot be interpreted! Moment curvature must be computed for larger plastic rotations.')
        print('         index_yy =' + str(index))

        # Determine outputs for the computed index
        ind_yy = index
        xP_yy = float(phi_yy_raw[index])
        yP_yy = float(Myy_raw[index])

        # Determine additional outputs
        M_ult_xx = float(max(Mxx))
        M_ult_yy = float(max(Myy))
        k_ult_xx = np.interp(M_ult_xx, Mxx, phi_xx)
        k_ult_yy = np.interp(M_ult_yy, Myy, phi_yy)

        # Combine results for x and y
        xP = np.array([abs(xP_xx), abs(xP_yy)])  # phi
        yP = np.array([abs(yP_xx), abs(yP_yy)])  # M
        M_ult = np.array([abs(M_ult_xx), abs(M_ult_yy)])
        k_ult = np.array([abs(k_ult_xx), abs(k_ult_yy)])
        ind = np.array([abs(ind_xx), abs(ind_yy)])

        return xP, yP, M_ult, k_ult, ind

    def getName(aData, filetype='title'):
        partType = 'Beam-RC'
        secType = aData.section_type
        if secType == "Rectangular Beam" or secType == 'T Beam':
            secType = 'Beam'
        elif secType == 'Circular' or secType == 'Rectangular Column':
            secType = 'Column'
        if filetype == '.key':  # .key file name
            filename = str(partType + '_' + secType + '_' +
                           str(aData.file_name) + '.key')
        else:  # PART naming within PRIMER
            filename = str(partType + '_' + secType +
                           ';' + str(aData.file_name))
        return filename

    def getPM(aData, n_strainlim, npoints):
        filename = aData.file_name
        pmxx_array, pmyy_array = pd.DataFrame(), pd.DataFrame()
        dat = pd.read_csv(aData.pSave + '\\' + str(filename) + '.txt',
                          delim_whitespace=True,
                          error_bad_lines=False,
                          skiprows=15)
        colstep = 38  # 38 columns added for each new PM curve
        colind = [12, 14, 31, 34]
        for i in range(0, n_strainlim):
            newind = [x + (colstep * i) for x in colind]
            pmxx, pmyy = dat.iloc[:, newind[0:2]], dat.iloc[:, newind[2:4]]
            pmxx.columns, pmyy.columns = [
                'P' + str(i), 'M' + str(i)], ['P' + str(i), 'M' + str(i)]
            pmxx_array = pd.concat([pmxx_array, pmxx], axis=1)
            pmyy_array = pd.concat([pmyy_array, pmyy], axis=1)
        pmxx_fin = pd.DataFrame(data=None, columns=['Px', 'Mx'])
        pmyy_fin = pd.DataFrame(data=None, columns=['Py', 'My'])

        # get envelope
        pmxx_fin = util.getEnvelope(pmxx_array, npoints, axis='xx')
        pmyy_fin = util.getEnvelope(pmyy_array, npoints, axis='yy')
        pm_envelope = pd.concat([pmxx_fin, pmyy_fin], axis=1)
        pm_all = pd.concat([pmxx_array, pmyy_array], axis=1)
        pm_all.to_csv(
            aData.pSave +
            '\\' +
            'Moment_Interaction' +
            filename +
            '.csv',
            index=False)
        pm_envelope.to_csv(aData.pSave + '\\' + 'Moment_Interaction_Envelope' +
                           filename + '.csv', index=False)
        pmxx_array.to_csv(
            aData.pSave +
            '\\' +
            'PMxx' +
            filename +
            '.csv',
            index=False)
        pmyy_array.to_csv(
            aData.pSave +
            '\\' +
            'PMyy' +
            filename +
            '.csv',
            index=False)

        return pm_envelope

    def getPlasticHingeLength(aData):
        shape = aData.section_type
        # Check plastic hinge length factor
        phl_xx = float(aData.phl_xx)
        phl_yy = float(aData.phl_yy)
        if float(phl_xx) < -1 or float(phl_yy) < -1:
            print('========================================')
            print('WARNING: plastic hinge length factor > 1')
            print('========================================')

        if shape == 'Circular':
            D = float(aData.diameter_circ) / 1000.0
            if phl_xx == 0 or phl_xx < -1:  # <-1 is a safety measure for factors > 1
                pl_hi_xx = D / 2  # default equations
            elif phl_xx > 0:  # absolute (return in m)
                pl_hi_xx = phl_xx / 1000.0
            elif phl_xx < 0:  # factor
                pl_hi_xx = D * abs(phl_xx)

            if phl_yy == 0 or phl_yy < -1:  # <-1 is a safety measure for factors > 1
                pl_hi_yy = D / 2  # defaulte quations
            elif phl_yy > 0:  # absolute
                pl_hi_yy = phl_yy / 1000.0
            elif phl_yy < 0:  # factor
                pl_hi_yy = D * abs(phl_yy)

        elif (shape == 'Rectangular Beam') | (shape == 'Rectangular Column'):
            # Get the width of the section (m)
            w = float(aData.rect_width) / 1000
            # Get the height of the section (m)
            h = float(aData.rect_height) / 1000
            # Plastic hinge length (m)
            if phl_xx == 0 or phl_xx < -1:  # <-1 is a safety measure for factors > 1
                pl_hi_xx = h / 2  # default equations
            elif phl_xx > 0:  # absolute (return in m)
                pl_hi_xx = phl_xx / 1000.0
            elif phl_xx < 0:  # factor
                pl_hi_xx = h * abs(phl_xx)

            if phl_yy == 0 or phl_yy < -1:  # <-1 is a safety measure for factors > 1
                pl_hi_yy = w / 2
            elif phl_yy > 0:  # absolute (return in m)
                pl_hi_yy = phl_yy / 1000.0
            elif phl_yy < 0:  # factor
                pl_hi_yy = w * abs(phl_yy)

        elif shape == 'T Beam':
            # Get the width of the flange (m)
            B = float(aData.f_width) / 1000
            # Get the height of the flange (m)
            h = float(aData.f_thick) / 1000
            # Get the width of the web (m)
            b = float(aData.t_width) / 1000
            # Get the height of the web (m)
            H = float(aData.t_height) / 1000 - float(aData.f_thick) / 1000
            if phl_xx == 0 or phl_xx < -1:  # <-1 is a safety measure for factors > 1
                pl_hi_xx = (H + h) / 2  # default equations
            elif phl_xx > 0:  # absolute (return in m)
                pl_hi_xx = phl_xx / 1000.0
            elif phl_xx < 0:  # factor
                pl_hi_xx = (H + h) * abs(phl_xx)

            if phl_yy == 0 or phl_yy < -1:  # <-1 is a safety measure for factors > 1
                pl_hi_yy = b / 2  # default equations
            elif phl_yy > 0:  # absolute (return in m)
                pl_hi_yy = phl_yy / 1000.0
            elif phl_yy < 0:  # factor
                pl_hi_yy = (b) * abs(phl_yy)

        elif shape == 'Cruciform':
            # Total width (m)
            bX = float(aData.bX) / 1000
            # Total height (m)
            bY = float(aData.bY) / 1000
            if phl_xx == 0 or phl_xx < -1:  # <-1 is a safety measure for factors > 1
                pl_hi_xx = bX / 2  # default equations
            elif phl_xx > 0:  # absolute (return in m)
                pl_hi_xx = phl_xx / 1000.0
            elif phl_xx < 0:  # factor
                pl_hi_xx = bX * abs(phl_xx)

            if phl_yy == 0 or phl_yy < -1:  # <-1 is a safety measure for factors > 1
                pl_hi_yy = bY / 2  # default equations
            elif phl_yy > 0:  # absolute (return in m)
                pl_hi_yy = phl_yy / 1000.0
            elif phl_yy < 0:  # factor
                pl_hi_yy = bY * abs(phl_yy)

        return pl_hi_xx, pl_hi_yy

    def getNormalize(dat):
        # get normalization factors
        pmax, pmin, mmax, mmin = 0, 0, 0, 0
        for i in range(0, len(dat.columns), 2):
            pmax_i = max(list(dat[dat.columns[i]]))
            mmax_i = max(list(dat[dat.columns[i + 1]]))
            pmin_i = min(list(dat[dat.columns[i]]))
            mmin_i = min(list(dat[dat.columns[i + 1]]))
            if pmax_i > pmax:
                pmax = pmax_i
            if mmax_i > mmax:
                mmax = mmax_i
            if pmin_i < pmin:
                pmin = pmin_i
            if mmin_i < mmin:
                mmin = mmin_i
        plen = abs(pmax) + abs(pmin)
        mlen = abs(mmax) + abs(mmin)

        return plen, mlen

    def getEnvelope(dat, nsteps, axis):
        # dat is the pandas dataframe that contains
        # the combined PM interaction curves for all limiting strains tested

        # nsteps is how many points you want on the final PM curve

        # step 1 - find max values to get the normalization values
        plen, mlen = util.getNormalize(dat)

        # step 2 - initialize the angles at which you want to generate the PM
        # envelope
        rad_vec = np.linspace(-np.pi, np.pi, nsteps)
        polar_array = np.zeros(shape=(nsteps, int(len(dat.columns) / 2.0 + 1)))
        polar_array[:, 0] = rad_vec

        # step 3 - loop through each limiting strain curve
        for i in range(0, len(dat.columns), 2):
            # normalize each limiting strain curve
            p = np.divide(list(dat[(dat.columns[i])]), plen)
            m = np.divide(list(dat[(dat.columns[i + 1])]), mlen)

            # polar coordinates
            rad = [math.atan2(p[x], m[x]) for x in range(0, len(p))]
            mag = [math.sqrt(pow(p[x], 2) + pow(m[x], 2))
                   for x in range(0, len(p))]
            polar_vec = pd.DataFrame({'T': rad, 'R': mag}).reindex(
                columns=['T', 'R'])  # constrain order of columns
            polar_sort = np.array(polar_vec.sort_values(by=['T']))
            polar_interp = interp1d(polar_sort[:,
                                               0],
                                    polar_sort[:,
                                               1],
                                    bounds_error=False,
                                    fill_value="extrapolate")(rad_vec)
            polar_array[:, int(i / 2.0 + 1)] = polar_interp

        # step 4 - now that we have polar coords for all curves at the same angles decided by rad_vec
        # compare the magnitude at each angle then pick max

        finalpm = np.zeros(shape=(nsteps, 2))  # 2 columns for P, M
        for r in range(0, len(polar_array[:, 0])):
            row = list(polar_array[r, 1:])
            # index of curve corresponding to the envelope curve
            ind = row.index(max(row))
            # now re-calculate the actual, non-normalized P and M values from
            # normalized polar coords
            p = plen * polar_array[r, ind + 1] * math.sin(polar_array[r, 0])
            m = mlen * polar_array[r, ind + 1] * math.cos(polar_array[r, 0])
            finalpm[r, :] = [m, p]
        if axis == 'xx':
            finalpm_df = pd.DataFrame(
                {'Px': finalpm[:, 1], 'Mx': finalpm[:, 0]})
        else:  # yy
            finalpm_df = pd.DataFrame(
                {'Py': finalpm[:, 1], 'My': finalpm[:, 0]})

        return finalpm_df

    def cpu_percent(proc):
        """CPU check for timing analysis"""

        # Check CPU usage of a process
        cpu_pct = 0.0
        cpu_pct = proc.cpu_percent()

        return cpu_pct

    def sample_cpu_percent(proc):
        """Flag when analysis is finished"""

        # Initialize
        stop_flag = 0
        cpu_hist = 0.0
        j = 0

        while j < 10000:
            cpu_pct = util.cpu_percent(proc)
            time.sleep(1)
            cpu_hist = max(cpu_pct, cpu_hist)
            j += 1

            if j > 10:
                if cpu_pct < 3:
                    stop_flag = 1
                    print('            Xtract analysis complete \n')
                    break

        return cpu_hist, stop_flag

    def addToClipBoard(text):

        df = pd.DataFrame([text])
        df.to_clipboard(index=False, header=False)



# -*- coding: utf-8 -*-
"""
Created on Thu Jul  5 11:52:55 2018

@author: kevin.stanton

Contains various mouse control and timing settings for hijacking XTRACT.
"""

import pandas as pd
import os

class settings:

    def __init__(self, aData):

        # Mouse position settings (change if mouse positions are incorrect)
        self.clickPosition1 = 60 # Click 'Project Name' - x
        self.clickPosition2 = 105 # Click 'Project Name' - y
        self.clickPosition3 = 125 # Click 'Show Data' - x
        self.clickPosition4 = 170 # Click 'Show Data' - y
        self.clickPosition5 = 700 # Click 'Select All' - x
        self.clickPosition6 = 1025 # Click 'Select All' or 'Save Data' - y
        self.clickPosition7 = 842 # Click 'Save Data' - x
        self.clickPosition8 = 1300 # Select dialog box at outermost level - x
        self.clickPosition9 = 400 # Select dialog box at outermost level - y
        
        # Time settings (all wait times in seconds, changes not necessary)
        self.sleepLaunch = 6  # Post-launch wait time
        self.sleepRunAnal = 4  # Post-execute XTRACT analysis command wait time
        self.sleepAt = 1.2  # Proxy for dynamically assigned additional sleep time while analysis is running
        self.sleepSLc = 2  # Proxy for data extraction sleep time
        self.sleepPaste = 1.0  # Post-paste cwd in Windows Explorer wait time
        self.sleepSave = 1.5  # Post-execute save command wait time

        # Compute timing constants (changes not necessary)
        self.sleepAt1 = 7 * self.sleepAt  # Analysis timing constant 1
        self.sleepAt2 = 5 * self.sleepAt  # Analysis timing constant 2
        self.sleepAt3 = 1.6 * self.sleepAt  # Analysis timing constant 3
        self.sleepSLc1 = 1.5 * self.sleepSLc  # Data extraction contant 1
        self.sleepSLc2 = 0.1 * self.sleepSLc  # Data extraction contant 2
        
        # Read in copy/paste behavior handling option
        self.controlMode = aData.controlMode 
        
        # Read in user-defined mouse click positions if specified
        if os.path.isfile(aData.settings_path):
            clickPositions = pd.read_csv(aData.settings_path, dtype=str, 
                                              header=None, sep=',\s+', 
                                              engine='python', index_col=0,
                                              comment='#')
            self.clickPosition1 = int(clickPositions.loc['clickPosition1']) # Click 'Project Name' - x
            self.clickPosition2 = int(clickPositions.loc['clickPosition2']) # Click 'Project Name' - y
            self.clickPosition3 = int(clickPositions.loc['clickPosition3']) # Click 'Show Data' - x
            self.clickPosition4 = int(clickPositions.loc['clickPosition4']) # Click 'Show Data' - y
            self.clickPosition5 = int(clickPositions.loc['clickPosition5']) # Click 'Select All' - x
            self.clickPosition6 = int(clickPositions.loc['clickPosition6']) # Click 'Select All' or 'Save Data' - y
            self.clickPosition7 = int(clickPositions.loc['clickPosition7']) # Click 'Save Data' - x
            self.clickPosition8 = int(clickPositions.loc['clickPosition8']) # Select dialog box at outermost level - x
            self.clickPosition9 = int(clickPositions.loc['clickPosition9']) # Select dialog box at outermost level - y
            

        


# -*- coding: utf-8 -*-
"""
Created on Wed Jan 29 13:59:31 2020

@author: kevin.stanton
"""

# Import shared libraries
import os
import sys
import pandas as pd
import numpy as np
import shutil
import time

# Import project libraries
from .util import util
from .construct_hysBeams import info_hysBeams
from .iXTRACT_envelope import iXTRACT_envelope
from .Shear_Calcs import Shear_Calcs
from .Autorun import Autorun
from .Mat_Hys_Beam import Mat_Hys_Beam
from .settings import settings
from .construct_elasticBeams import info_elasticBeams
from .Elastic_Beam import Elastic_Beam
from .beamGeometry import beamGeometry

def MHB(iSet=1,
        pInputs=os.path.join(os.getcwd(),'inputs_hysBeams.xlsx'),
        pSave=os.path.join(os.getcwd(),'Output_hysBeams'),
        codeOption='EC2',
        concreteFactor='normal',
        fParams='ASCE 41-17',
        tbOption='default',
        concrete_strain_limits=[
            0.0012,
            0.0024,
            0.0035],
        steel_strain_limits=[
            1.0,
            1.5],
        npoints=401,
        mphisolution='BiSection',
        bilinearflag=False,
        cLimits='explicit',
        cleanCurves=True,
        skipFlag=True,
        pXTRACT=r'C:\Program Files (x86)\TRC\XTRACT\XTRACT.exe',
        mInter=False,
        timingControl='automatic',
        controlMode='default',
        author='user',
        company='Arup',
        job_name='dev',
        showPlots=True,
        write_log=False,
        user_xpj=False,
        settings_path=''):
    
    
    """
    This function intializes an object to enable individual function testing 
    outside of the main pipeline for MAT_HYSTERETIC_BEAM card writing.
    """
    
    
    # Turn off batch mode
    batchMode=False
    
    # Check that inputs file exists and handle if not
    if os.path.isfile(pInputs) == False:
        raise AssertionError(
            'Inputs file not found: ' + str(pInputs))

    # Parse input file
    # Get input file type
    ext = os.path.splitext(pInputs)[1]
    # Specify interpreter order
    iData_order = ['info_filename',
                   'Fc',
                   'Ft',
                   'ey',
                   'ecu',
                   'esp',
                   'ef',
                   'Ec',
                   'den',
                   'v',
                   'Fy',
                   'Fu',
                   'esh',
                   'esu',
                   'Es',
                   'section_type',
                   'cover',
                   'diameter_circ',
                   'bar_diam_circ',
                   'num_bars_circ',
                   'rect_width',
                   'rect_height',
                   'n_t',
                   'n_b',
                   'db_t',
                   'db_b',
                   'num_bars_col',
                   'bar_diam_col',
                   't_width',
                   't_height',
                   'f_width',
                   'f_thick',
                   'f_lay',
                   'f_bar',
                   'f_spac',
                   't_nt',
                   't_tbar',
                   't_nb',
                   't_bbar',
                   'bX',
                   'bY',
                   'tX',
                   'tY',
                   'cX',
                   'cY',
                   'cBar_centerD',
                   'cBar_vFlangeD',
                   'cBar_hFlangeD',
                   'cBar_vFlangeN',
                   'cBar_hFlangeN',
                   'cBar_rows',
                   'M_load',
                   'N_load',
                   'V_load',
                   'd_shear',
                   's_shear',
                   'n_legs',
                   'gc_mat',
                   'gs_mat',
                   'Fywd',
                   'phl_xx',
                   'phl_yy',
                   'rb_condition',
                   'c_condition',
                   'ss_ratio_s',
                   'ss_ratio_t']

    # Read data
    if ext == '.txt':
        iData = pd.read_csv(pInputs, dtype=str, header=None,
                            sep=',\s+', engine='python')
    elif ext == '.csv':
        iData = pd.read_csv(pInputs, dtype=str, header=None)
    elif ext == '.xlsx':
        iData = pd.read_excel(pInputs, dtype=str, header=None)
    # Remove whitespaces from strings
    iData = iData.applymap(lambda x: x.strip() if isinstance(x, str) else x)
    # Remove data rows not spacified in iData_order
    iData = iData[iData.iloc[:, 0].isin(iData_order)]
    # Remove second column if full input format detected
    if 'List' in iData.iloc[0, 1]:
        iData = iData.drop(columns=1)
    # Re-index
    iData = iData.set_index(0)
    # Sort according to iData_order
    iData = iData.loc[iData_order]
    # Re-index columns to match iRun handles
    iData.columns = pd.RangeIndex(0, len(iData.columns))

    # Tag output folder with current time
    timestr = time.strftime("%Y%m%d-%H%M%S")
    pSave = str(pSave + '_' + timestr)
    os.makedirs(pSave)

    # Append static inputs to data array
    XTRACT = 'on'
    devMode = 'on'
    envelopeflag = True  # use PM enveloping approach
    sData = [
        author,
        company,
        job_name,
        pXTRACT,
        mInter,
        batchMode,
        iSet,
        fParams,
        tbOption,
        devMode,
        cLimits,
        cleanCurves,
        showPlots,
        controlMode,
        codeOption,
        concreteFactor,
        timingControl,
        pSave,
        write_log,
        settings_path]
    data = iData, sData
    
    iRun = iSet - 1
    
    aData = info_hysBeams(data, iRun)
    
    return aData


# -*- coding: utf-8 -*-
"""
Created on Tue May 15 12:23:40 2018

@author: kevin.stanton
"""

import math
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from scipy.optimize import curve_fit
from scipy.signal import savgol_filter


def biLin(
        M_phi,
        sectionName,
        plot_fig=False,
        constrained=False,
        cleanCurves=False):
    '''
    This method assigns a 2-stage piecewise linear curve fit to 2D array data.

    Inputs:
        M_phi = DataFrame of moment curvature values for x and y directions
        a, b = Guesses for slope (a) and intercept (b) on segment P1 (after yield)
        c, d = Guesses for slope (c) and intercept (d) on segment P2 (before yield)
    '''

    def pSegs(x, a, b, c, d):
        '''
        Function to compute piecewise segments.
        '''

        P1 = a * np.array(x) + b
        P2 = c * np.array(x) + d

        return np.minimum(P1, P2)

    def cutOutlier(x, y):

        i = 0
        xMod = []
        yMod = []
        while i < len(y) - 1:
            if float(y[i]) < float(y[i + 1]):
                xMod.append(x[i])
                yMod.append(y[i])
            i = i + 1

        return xMod, yMod

    def preFit(x, y):

        i = 1
        while i < len(y):
            if float(y[i]) == max(y):
                c = i
            i = i + 1

        x = x[:c]
        y = y[:c]

        return x, y

    def refineFit(x, y, index):

        i = 1
        c = len(y)
        while i <= len(y):
            if float(y[i]) > index:
                c = min(i + 4, len(y))
                i = len(y)
            i = i + 1

        x = x[:c]
        y = y[:c]

        return x, y

    ######################################
    ########### Bi-linearize: x ##########
    ######################################

    # Read in data
    y, x = M_phi.loc[:, 'Mxx (N-m)'], M_phi.loc[:, 'phi_xx (1/m)']

    # Remove outliers and prepare for the fitting process
    x, y = cutOutlier(x, y)
    xMod, yMod = preFit(x, y)

    # Set constants for initaial bi-linear guess
    a = max((yMod[len(yMod) - 1] - yMod[len(yMod) - 2]) /
            max(xMod[len(xMod) - 1] - xMod[len(xMod) - 2], 0.0001), 10)
    b = max(yMod) * 0.8
    c = (yMod[2] - yMod[1]) / (xMod[2] - xMod[1])
    d = 0
    if c < 0 or math.isnan(c):
        c = b / (max(x) / 4)

    # Fit piecewise function to identify the preliminary yield point
    pw0 = (a, b, c, d)
    pw, cov = curve_fit(pSegs, xMod, yMod, pw0)

    # Define preliminary yield point
    xP_xx = (pw[3] - pw[1]) / (pw[0] - pw[2])
    yP_xx = min(max(yMod), abs(
        pw[0] * ((pw[3] - pw[1]) / (pw[0] - pw[2])) + pw[1]))

    # Refine interpreation of the yield point
    a = (max(yMod) - yP_xx) / (max(xMod) - xP_xx)
    b = pw[1]
    c = yP_xx / xP_xx
    xMod, yMod = refineFit(xMod, yMod, yP_xx)
    sigma = np.ones(len(xMod))
    pw0 = (a, b, c, d)
    if constrained:
        sigma[[0, -1]] = 0.01
    pw, cov = curve_fit(pSegs, xMod, yMod, pw0, sigma=sigma)
    xP_xx = (pw[3] - pw[1]) / (pw[0] - pw[2])
    yP_xx = pw[0] * ((pw[3] - pw[1]) / (pw[0] - pw[2])) + pw[1]

    # Determine ultimate moment capacity coordinates
    M_ult_xx = max(y)
    k_ult_xx = np.interp(
        M_ult_xx, M_phi.loc[:, 'Mxx (N-m)'], M_phi.loc[:, 'phi_xx (1/m)'])

    # Convert result back to DF
    M_phi_biLin_xx = pd.DataFrame(
        {'Mxx (N-m)': pSegs(x, *pw), 'phi_xx (1/m)': x})

    # Remove non-monotonically increasing values
    M_phi_biLin_mI_xx = np.maximum.accumulate(M_phi_biLin_xx.values)

    M_phi_biLin_xx = pd.DataFrame(
        {'Mxx (N-m)': M_phi_biLin_mI_xx[:, 0], 'phi_xx (1/m)': M_phi_biLin_mI_xx[:, 1]})

    M_phi_biLin_xx = M_phi_biLin_xx.drop_duplicates()

    # =========================================================================
    # Plot results for sanity check
    if not cleanCurves:
        y, x = M_phi.loc[:, 'Mxx (N-m)'], M_phi.loc[:, 'phi_xx (1/m)']

    if plot_fig:
        plt.figure(1)
        plt.title('M_phi_xx')
        plt.plot(x, y, 'o', x, pSegs(x, *pw), '-')
        plt.plot(xP_xx, yP_xx, color='red', marker='^')
        plt.savefig('M_phi_xx-' + str(sectionName) + '.png')
        plt.show()

    # =========================================================================

    ######################################
    ########### Bi-linearize: y ##########
    ######################################
    y, x = M_phi.loc[:, 'Myy (N-m)'], M_phi.loc[:, 'phi_yy (1/m)']
    x, y = cutOutlier(x, y)
    xMod, yMod = preFit(x, y)

    # Set constants for initaial bi-linear guess
    a = max((yMod[len(yMod) - 1] - yMod[len(yMod) - 2]) /
            max(xMod[len(xMod) - 1] - xMod[len(xMod) - 2], 0.0001), 10)
    b = max(yMod) * 0.8
    c = (yMod[2] - yMod[0]) / (xMod[2] - xMod[0])
    if c < 0 or math.isnan(c):
        c = b / (max(x) / 4)

    # Fit piecewise function to identify the preliminary yield point
    pw0 = (a, b, c, d)
    pw, cov = curve_fit(pSegs, xMod, yMod, pw0)

    # Define preliminary yield point
    xP_yy = (pw[3] - pw[1]) / (pw[0] - pw[2])
    yP_yy = min(max(yMod), abs(
        pw[0] * ((pw[3] - pw[1]) / (pw[0] - pw[2])) + pw[1]))

    # Refine interpreation of the yield point
    a = (max(yMod) - yP_yy) / (max(xMod) - xP_yy)
    b = pw[1]
    c = yP_yy / xP_yy
    xMod, yMod = refineFit(xMod, yMod, yP_yy)
    sigma = np.ones(len(xMod))
    pw0 = (a, b, c, d)
    if constrained:
        sigma[[0, -1]] = 0.01
    pw, cov = curve_fit(pSegs, xMod, yMod, pw0, sigma=sigma)
    xP_yy = (pw[3] - pw[1]) / (pw[0] - pw[2])
    yP_yy = pw[0] * ((pw[3] - pw[1]) / (pw[0] - pw[2])) + pw[1]

    # Determine ultimate moment capacity coordinates
    M_ult_yy = max(y)
    k_ult_yy = np.interp(
        M_ult_yy, M_phi.loc[:, 'Myy (N-m)'], M_phi.loc[:, 'phi_yy (1/m)'])

    # Convert result back to DF
    M_phi_biLin_yy = pd.DataFrame(
        {'Myy (N-m)': pSegs(x, *pw), 'phi_yy (1/m)': x})

    # Remove non-monotonically increasing values
    M_phi_biLin_mI_yy = np.maximum.accumulate(M_phi_biLin_yy.values)
    M_phi_biLin_yy = pd.DataFrame(
        {'Myy (N-m)': M_phi_biLin_mI_yy[:, 0], 'phi_yy (1/m)': M_phi_biLin_mI_yy[:, 1]})
    M_phi_biLin_yy = M_phi_biLin_yy.drop_duplicates()

    # =========================================================================
    # Plot results for sanity check
    if not cleanCurves:
        y, x = M_phi.loc[:, 'Myy (N-m)'], M_phi.loc[:, 'phi_yy (1/m)']

    if plot_fig:
        plt.figure(2)
        plt.title('M_phi_yy')
        plt.plot(x, y, 'o', x, pSegs(x, *pw), '-')
        plt.plot(xP_yy, yP_yy, color='red', marker='^')
        plt.savefig('M_phi_yy-' + str(sectionName) + '.png')
        plt.show()

    # =========================================================================

    #########################################
    ###### Combine results for x and y ######
    #########################################

    if len(M_phi_biLin_xx.dropna()) < len(M_phi_biLin_yy.dropna()):
        M_phi_biLin = pd.concat([M_phi_biLin_xx, M_phi_biLin_yy], axis=1, join_axes=[
                                M_phi_biLin_xx.index])
    else:
        M_phi_biLin = pd.concat([M_phi_biLin_xx, M_phi_biLin_yy], axis=1, join_axes=[
                                M_phi_biLin_yy.index])

    xP = np.array([abs(xP_xx), abs(xP_yy)])  # phi
    yP = np.array([abs(yP_xx), abs(yP_yy)])  # M
    M_ult = np.array([abs(M_ult_xx), abs(M_ult_yy)])
    k_ult = np.array([abs(k_ult_xx), abs(k_ult_yy)])

    return xP, yP, M_phi_biLin, M_ult, k_ult



# -*- coding: utf-8 -*-
"""
Created on Tue Nov 13 11:22:24 2018

@author: kevin.stanton
"""

from setuptools import setup, find_packages

with open("ReadMe.md", "r") as fh:
    long_description = fh.read()

setup(name='autodyna',
      version='2.4.4',
      description='A collection of LS-DYNA card writing tools',
      long_description=long_description,
      long_description_content_type="text/markdown",
      url='https://gitlab.arup.com/kevin.stanton/AutomatedCardGenerator_LS-DYNA',
      author='Kevin Stanton',
      author_email='kevin.stanton@arup.com',
      install_requires=['matplotlib',
                        'numpy',
                        'pywin32',
                        'pandas',
                        'datetime',
                        'psutil',
                        'scipy'],
      packages=find_packages(),
      data_files=[('autodyna',
                   ['autodyna/inputs_elasticBeams.xlsx',
                    'autodyna/inputs_elasticBeams.csv',
                    'autodyna/inputs_elasticBeams.txt',
                    'autodyna/inputs_hysBeams.xlsx',
                    'autodyna/inputs_hysBeams.csv',
                    'autodyna/inputs_hysBeams.txt',
                    'autodyna/settings.txt']),
                  ('autodyna/Array_Formatting',
                   ['autodyna/Array_Formatting/FormatForDYNA.xlsm',
                    'autodyna/Array_Formatting/ReadMe.md']),
                  ('autodyna/Resources/XTRACT',
                   ['autodyna/Resources/XTRACT/iai.key',
                    'autodyna/Resources/XTRACT/SmallXTRACT.ico',
                    'autodyna/Resources/XTRACT/XTRACT.cnt',
                    'autodyna/Resources/XTRACT/XTRACT.exe',
                    'autodyna/Resources/XTRACT/XTRACT.GID',
                    'autodyna/Resources/XTRACT/atl.dll',
                    'autodyna/Resources/XTRACT/checkKey.dll',
                    'autodyna/Resources/XTRACT/GDI32.DLL'])],
      include_package_data=True)
